 use strict;
 use vars qw(@PhotConf %PhotCVals);

my $PhotConf='photcal.conf';



 sub InitPhotCal {

 @PhotConf=(

    {
   COMM =>  ' ==== DIRECTORIES',
    }, 

    {
   KEY  =>  'HDR_DIR',
   COMM =>  ' The directory containing headers',
    }, 

    {
   KEY  =>  'FITS_DIR',
   COMM =>  ' The directory containing the FITS files',
    }, 

#    {
#   KEY  =>  'XCATS_DIR',
#   COMM =>  ' The directory containing the SExtractor catalogs',
#    }, 

    {
   KEY  =>  'OUTDIR',
   COMM =>  ' The output directory',
    }, 

    {
   COMM =>  ' ====  The I/O keyword',
    }, 

    {
   KEY  =>  'LIST',
   COMM =>  ' Input lists separated by commas',
    }, 


   {
   KEY  =>  'STDTAB',
   COMM =>  ' Standard field table (one per list entry)',
    }, 



   {
   KEY  =>  'SUM_FILE',
   COMM =>  ' The name of the summary file',
    },
  

    {
   COMM =>  ' ====  Settings',
    },

   {
   KEY  =>  'BAND',
   COMM =>  ' The photometric band to be calibrated (UBVRI)',
    },
 

  {
   KEY  =>  'EXPNAME',
   COMM =>  'The PERL regexp to separate the root and CCD number   ',
    }, 


  {
   KEY  =>  'EXTC',
   COMM =>  'Initial guess of the extinction coefficient   ',
    }, 

  {
   KEY  =>  'FIX_EXTC',
   COMM =>  'Keep constant the extinction coefficient ?   ',
    }, 




   {
   KEY  =>  'MAG_APERTURE',
   COMM =>  'The aperture diameter (pixels) used by SExtractor ',
    }, 
 
   {
   KEY  =>  'THRESHOLD',
   COMM =>  'The threshold used by SExtractor ',
    }, 
 
   {
   KEY  =>  'MATCH_RADIUS',
   COMM =>  'The  radius (arcsec) of the search box to match catalogs',
    }, 

   {
   KEY  =>  'SIGMA',
   COMM =>  'Sigma clipping in the interation',
    }, 
 

   {
   KEY  =>  'MAXITER',
   COMM =>  'Maximum number of iterations in the fit',
    }, 

   {
   KEY  =>  'MINCHI2',
   COMM =>  'Relative phot.: minimum chi**2 in residuals',
    }, 



    {
   COMM =>  ' ====  FITS header keywords',
    }, 

   {
   KEY  =>  'INSTRUMENT',
   COMM =>  'The name of the instrument (CFH12K, ESO,....) ',
    }, 

   {
   KEY  =>  'K_DATE',
   COMM =>  'The name of the keyword with DATE ',
    }, 

   {
   KEY  =>  'K_TIME',
   COMM =>  'The name of the keyword with TIME ',
    }, 


   {
   KEY  =>  'K_EXPTIME',
   COMM =>  'The name of the keyword with EXPTIME ',
    }, 

   {
   KEY  =>  'K_AIRMASS',
   COMM =>  'The name of the keyword with AIRMASS ',
    }, 



     );

# Here values are stored
%PhotCVals = (
 LIST => [],
 STDTAB => [],
 MAG_APERTURE => 25,
 THRESHOLD => 10,
 MATCH_RADIUS => 1,
 MAXITER => 3,
 SIGMA => 1.5,
 EXTC => 0,
 FIX_EXTC => 'N',
 MINCHI2 => 0.001,
 SUM_FILE => 'photcal.dat',
 BAND => 'U',
 EXPNAME => '(.*\D)(\d+)(\D+)$', 
 INSTRUMENT => 'CFH12K',
 K_DATE => 'DATE-OBS',
 K_TIME => 'UTC-OBS', 
 K_EXPTIME => 'EXPTIME',
 K_AIRMASS => 'AIRMASS'
    );

#my %ObsPars =(
##entries: latitude, longitude
# ESO => ['-29:15.4','70:43.8'],
# CFHT => ['19:49.6','155:28.3']
#)

     };

sub mkPhotConf
{

my $pfile;

if ($#_ >= 0) {$pfile=shift}
else {$pfile=$PhotConf}
#$pfile="del";

open (FILE, ">$pfile"); 
my $entry;
foreach $entry (@PhotConf) {
  my $nokey=0;      
   if (exists $entry->{KEY}) {
      my $key=$entry->{KEY}; 
      if (exists $PhotCVals{$key}) {
         my $val=$PhotCVals{$key};
         if (ref($val) eq 'ARRAY') {
              my @arr=@{$val};
	      if ($#arr==-1) {print FILE "$key\t\t"}
              foreach my $arr (@arr) {
                 print FILE "$key\t $arr\n";  
              }
	 } else {               
           print FILE "$key\t $PhotCVals{$key}\t";
           } 
	} 
        else {print FILE "$key\t\t" } 
    } else {$nokey=1}
    if (exists $entry->{COMM}) {
        my $comm=$entry->{COMM};
         if ($nokey==1 && $comm !~ '--') {print FILE "\t\t\t"}
	 
           print FILE "#$comm";
	}   
  print FILE "\n";	
   }
   
 close (FILE);  
 }


sub ReadPhotConf
{

my $pfile;

if ($#_ >= 0) {$pfile=shift}
else {$pfile=$PhotConf}

return if (! -e $pfile);

open (FILE, "<$pfile"); 

while (<FILE>) {
  chomp;

  next if ($_ =~ /^(\s*|\t*)\#/); 
  my ($vals, $comm)=split(/\#/, $_);
  my ($key, $val)=split(/\s+|\t+/, $vals);

 #  next if (! exists $PhotCVals{$key});

  if ($key eq 'LIST' || $key eq 'STDTAB') {my @all=@{$PhotCVals{$key}};
     push @all, $val;
     $PhotCVals{$key}=[@all];

   } 
  else {$PhotCVals{$key} = $val}
 }

 close (FILE);  
 }


