use strict;
use Tie::IxHash;
use Time::Local;

use Cwd;

use vars qw($INSTRUMENT $IMPL_INSTR  $SOFT $REDO $CLEAN $TFILTER $DEBUG $PROC $MAXMEM);
use vars qw($BIN $VERSION $PIPEFILE $WORK_DIR $DATADIR $OUTDIR $STOREDIR $PSDIR $CONFIG
$CONFEXT $PREFS $TMPDIR $BIN $LOGFILE $TMPLOGFILE $SFILE);
use vars qw($SDFILE $DBINDIR $DBOUTDIR $DBDEL);
use vars qw($SExbin $SExConf $SExPars $SExConv);
use vars qw($WWbin $WWConf);


sub Init
{

#--------- The SOFTWARE

$SOFT = {
    NAME => ['SOFTNAME', 'Prerepix',
              'The software that processed the data'],	      
    VERSION => ['SOFTVERS', '0.97d', 'Version of the software'],
    DATE    => ['SOFTDATE', '2001-05-12','Release date of the software'],
    AUTHOR  => ['SOFTAUTH', 'Mario Radovich', 'Maintainer of the software'],
    INST    => ['SOFTINST', 'TERAPIX team at IAP  http://terapix.iap.fr', 
                       'Institute']
   };		       	      

$WORK_DIR=cwd;
$OUTDIR = "Processed/";           #output directory - main level
$STOREDIR = $OUTDIR.'Store/';     #output directory - final results
$PSDIR = $OUTDIR.'Plots/';        #output directory - postscript plots
$DATADIR = 'Data/';
$TMPDIR = $OUTDIR.'Temp/';        #output directory - temporary results
$CONFIG = "prered";            #the name of the configuration file
$CONFEXT="par";                #the extension for the configuration files
$PREFS ='prerepix.conf';          # the root of the name of the file storing the setting
$MAXMEM=200;                      # The maxmimum amount of memory for combination (Mb)
$INSTRUMENT = 'CFH12K';           # The default instrument
$IMPL_INSTR = ['CFH12K','ESOWFI', 'UH8K'];  #the implemented instruments

$LOGFILE = "";                    #The name of the logfile (will be filled later)
$TMPLOGFILE = "";
$PIPEFILE = ".pcomm" ;            # The name of the file used for piping commands
                                  # could be replaced by an auto tmp....
$SFILE = $OUTDIR.'summary.txt';

# PREFERENCES FOR THE DATABASE
$SDFILE = "setdata.db";           #The default name of the database file
$DBINDIR=cwd;
$DBOUTDIR=cwd;
$DBDEL = "\t";   #the delimiter in the db file

my $n;
($n, $BIN)=fileparse($0,'.pl');   #The path of the executable

$DEBUG = 0;                     #Put 1 for verbose mode


# Here some features of the supported instruments are defined

$TFILTER = 1;      #if 1, then skip an image in case of filter mismatch
$REDO = 'no';
$CLEAN = 'no';

# ------------------ The SExtractor parameters 
$SExbin='sex';
$SExConf='sex.conf';
$SExPars='sex.pars';
$SExConv='sex.conv';

# ------------------ The WeightWatcher parameters 
$WWbin='ww';
$WWConf='ww.conf';


}

sub LoadPrefs
{

return if (! open (OUT, "$PREFS"));
my @s=split(/,/,<OUT>);
close (OUT);
foreach my $s (@s) {
  my @v=split(/:/, $s, 2);
  next if ($#v != 1); 
  eval ("\$$v[0] = $v[1]");
 }
}

sub SavePrefs
{

  if (! open (OUT, ">$PREFS"))  {
        warn "Can't open preferences' file $PREFS !"; 
        return 1
   }

my $string=join ',', 
       "MAXMEM:$MAXMEM",
       "REDO:'$REDO'",
       "CLEAN:'$CLEAN'",
       "SDFILE:'$SDFILE'",
       "DBINDIR:'$DBINDIR'",
       "DATADIR:'$DATADIR'",
       "INSTRUMENT:'$INSTRUMENT'",
       "SExbin:'$SExbin'",
       "WWbin:'$WWbin'",
       "DEBUG:$DEBUG";
printf OUT $string;
close (OUT);
}


sub mkdate {
   #gives as output the date and time 
   
   my @lt=localtime;

   my $date=sprintf("%d-%02d-%02d", $lt[5]+1900, $lt[4]+1,$lt[3]);
   my $time=join (':', $lt[2], $lt[1], $lt[0]);

   my $today=$date.'T'.$time;
   
      return $today;
} #end sub mkdate

sub mkloctime
  {
# convert date to seconds from 1/1/1970
my $hdate=shift;
my ($date, $time)=split(/T/,$hdate);
my @date=split('-', $date);
my @time=split(':', $time);

push @time, 0 if ($#time<2);

my $lt=timelocal($time[2], $time[1], $time[0], $date[2], $date[1]-1, $date[0]);

return $lt;

  }


sub mktList

{
my $inlist=shift;
my $outlist;

 my $plist=$inlist;
  #strip the @
  $plist=~s/\@//;
  my $nopath=$plist;
  #take only filename without path
  my @string=reverse(split(/\//,$nopath));
  $nopath=$string[0];
  # add out-
  $nopath="out-".$nopath;
  #substitute xxx with out-xxx in the complete filename
  $outlist=$plist;
  #$outlist=~ s/$string[0]/$nopath/;
   $outlist=$TMPDIR.$nopath;
  return $outlist;
}

1;
