#!/usr/local/bin/perl

#use English;

package get_params_flips;

#use Tk;
#use Tk::Dialog;
#use Tk::Table;
use Common::FileDialog;
use Flips::help;

use strict;


use vars qw/$task $inlist $outlist $proc $go $mainfile $fname @frames $hdlg/;
use vars qw /$tpfile $satur/;
use vars qw/@nfields %def_vals/; #the parameters hash


#------------------------------------------
# Phase 0: create the front end.
#------------------------------------------

sub mkform {

$task=shift;
$inlist=shift;
my $routlist=shift;

$outlist="";

my $ret=0;

&build_name($task);


if ($main::redmode ne "interactive") {mkConfig($task);}

# This is the file to be used as an "archive" of parameter 
$fname=$tpfile;

my $app="";

#The form for each task is only displayed in interactive mode, or in test mode if the debug flag is set
if ($main::interface eq 'gui' && ($main::redmode eq "interactive" || ($main::redmode eq "listonly" && $main::debug eq 'y') ) ) {


&flips_help::defHelp($task);

$app = $main::top->Toplevel;
$app->title('FLIPS interface: '.$task);

# Use a scrollable table (useful for task with many parameters)
my $t  = $app->Table(-scrollbars => 'e',-fixedrows => 1, -rows => 20);
$t->pack(-expand=> 1, -fill => 'both');

my $c=0;

$t->put(0,$c++,$t->Label(-text=>"Class",-anchor=>'w'),-relief => 'raised');
$t->put(0,$c++,$t->Label(-text=>"Field",-anchor=>'w'),-relief => 'raised');
$t->put(0,$c++,$t->Label(-text=>"Value",-anchor=>'w'),-relief => 'raised');

$t->configure(-columns => $c) if ($c > $t->cget('-columns'));



@frames=();

my $r=1;

foreach my $field (keys %def_vals) {

    my $c=0;

    $field=uc($field);

    for my $entry (keys %{ $def_vals{$field}} ) {
 
       my $e;
   
       ${$e} = $t->Entry(-width => '50', -relief => 'sunken');

       $frames[$r]=${$e};

       $t->put($r,$c++,$t->Label(-text => "$field", -anchor => 'e'));
       $t->put($r,$c++,$t->Label(-text => "$entry", -anchor => 'e'));
       $t->put($r,$c++,${$e});

       $r++;
    }
}


my $buttons = $app->Frame();
$buttons->pack(-side => 'bottom', -pady => '2', -anchor => 'center');
my $buttons_clear = $buttons->Button(-text => 'Clear');
my $buttons_load = $buttons->Button(-text => 'Load');
my $buttons_save = $buttons->Button(-text => 'Save as');
my $buttons_savedef = $buttons->Button(-text => 'Save def.');
my $buttons_go = $buttons->Button(-text => $go);
my $buttons_skip = $buttons->Button(-text => 'Next');

#If not interactive mode, windows are only used to show the parameters (read only)
if ($main::redmode eq "interactive") {
    $buttons_clear->pack(-side => 'left', -padx => '2');
    $buttons_load->pack(-side => 'left', -padx => '2');
    $buttons_save->pack(-side => 'left', -padx => '2');
    $buttons_savedef->pack(-side => 'left', -padx => '2');
    $buttons_go->pack(-side => 'left', -padx => '2');
    $buttons_skip->pack(-side => 'left', -padx => '2');
  }

#------------------------------------------
# Phase 1: Add menus, dialog boxes
#------------------------------------------


my $menu = $app->Frame(-relief => 'raised', -borderwidth => '1');
$menu->pack(-before => $t, -side => 'top', -fill => 'x');

my $menu_file = $menu->Menubutton(-text => 'File', -underline => 0);

$menu_file->command(-label => 'Exit', -command => sub{$ret=1;$app->destroy}, -underline => 0);
$menu_file->pack(-side => 'left');



# ------------------------------------------
# Phase 2: Help section
#------------------------------------------

$hdlg = $app->Dialog(-title => 'Help',-justify => 'left');


$app->bind('<Any-F12>' => sub {
    my $w = shift;
    my $e = $w->XEvent;
    &msgHelp($w->containing($e->X, $e->Y), $e->X, $e->Y);
});


#------------------------------------------
# Phase 4: Miscellaneous other actions
#------------------------------------------


$buttons_clear->configure(-command => \&clearAction);

$buttons_go->configure(-command => \&goAction);


$buttons_load->configure(-command => sub { &fileAction('load')} );
$buttons_save->configure(-command => sub{ &fileAction('save')});
$buttons_savedef->configure(-command => sub{&SaveParams($tpfile)});
$buttons_skip->configure(-command => sub{$$routlist=$outlist;$app->destroy});

my $menu_file_m = $menu_file->cget(-menu);
$menu_file_m->entryconfigure(2, -accelerator => 'Ctrl+Q');
$app->bind('<Control-q>' => sub {exit});

if ($main::redmode ne "auto") {LoadParams($tpfile)};

# We do this rather than simply $frame_1_e->focus to eliminate the Perl
# "single use" warning message.

$frames[1]->focus;

 #end of if not AUTO
}

#wait for user only in interactive mode
if ($main::interface eq 'gui' && $main::redmode eq "interactive") {$app->waitWindow} else {$ret=&goAction}



$$routlist=$outlist;

return $ret;

}

sub clearAction {
    foreach my $i (0..$#nfields) {
	$frames[$i+1]->delete('0', 'end');
    }
}  #end clearAction

sub goAction {

     use Shell qw(flipsrun);

my $elapsed=time;
my @result=();

  use File::Copy;

     my $ret=0;


     SaveParams($mainfile, "go"); # FLIPS needs this file....   
                                  # The "go" means that the existence of lists
                                  # will be checked
     if ($inlist eq "") {$ret=1 ;print "$task was not executed \n";return $ret}


#---- Initialize the 'timer' variables -------
     my ($wclock0, $cpu0);
    if ($main::redmode ne "listonly") {
         $wclock0=time;
         $cpu0=(times)[0];
    }
#-------------------------------------------

#---- run FLIPS with the required tasks and parameters
   if ($task =~ /red$/) {
     if ($main::redmode ne "listonly") {
          @result=flipsrun ("red", "0")} else { print "$task: $inlist -> $outlist \n";}
    }
  elsif ($task =~ /comb$/) {  
# make input and output list files        
     if ($main::redmode ne "listonly") { 
            &lmod2comb($inlist,$outlist,$satur);
             @result=flipsrun ("combine", "0");
       } else { print "$task: $inlist -> $outlist \n";}
  }
  elsif ($task =~ /mode$/ || $task =~ /norm$/) {  
     if ($main::redmode ne "listonly") {
              @result=flipsrun ("mode", "0");
              if ($task =~ /norm$/) {copy("immode.dat","flatmode.dat")}		    
	    } else { print "$task: $inlist\n"};
          }	  
   if ($main::redmode ne "listonly") {
            my $cpu1=(times)[0];
            my $wclock1 = time;
         #print "$result\n"; #This may be sento to a logfile.......
	 #open (FILE, ">>$main::logfile"); 
           # print FILE "@result\n";
	 #close (FILE);   
         printf "FLIPS: $task COMPLETED, it took:  %.2fs (user), %.2fs (CPU)\n", $wclock1-$wclock0,$cpu1-$cpu0;

    }


  my @errs=grep(/exiting/, @result);
    my $nerr=$#errs+1;


# if ($nerr != 0 || $nwarn != 0) {

 if ($nerr != 0) {

    open(OFILE, ">> $main::logfile");

         print OFILE "$nerr ERRORS\n";
           print OFILE "@result\n";
         #foreach my $err (@errs) {
         #   my @string=split(/exiting/,$err,2);
         #   print OFILE "$string[1]\n";
         # 
     
      
 #      if ($nwarn > 0) {
 #        print OFILE "$nwarn WARNINGS\n";
 #        foreach my $warn (@warns) {
 #           my @string=split(/Warning:/,$warn,2);
 #           print OFILE "$string[1]\n";
 #         }
     
      }
          	  
     close (OFILE);
 
  return $nerr;

} #end goAction

sub fileAction {
    my $oper=shift;
    &main::getFile($oper,\$fname);
    #print "loading parameters from file $fname \n";
    if ($oper eq 'load') {LoadParams($fname, ".param");}
    if ($oper eq 'save') {SaveParams($fname, ".param");}
} #end fileAction




sub LoadParams
  {

if ($main::redmode eq "listonly") {

     my $i=0;

     foreach my $field (keys %def_vals) {
        for my $entry (keys %{ $def_vals{$field}} ) {
           $i++; 
           $frames[$i]->delete('0','end');
           $frames[$i]->insert('0',$def_vals{$field}{$entry});
          }
    }
  return;
}



 my $file = shift;

&clearAction; #clear the window

if ( not open (FILE, $file)) {
	$main::top->Dialog(
            -title  => 'File Not Found',
            -text   => "Can't load $file",
            -bitmap => 'error',
        )->Show;
	return
     }

#read parameter file
my $fclass="";

while (<FILE>) {
 chomp($_);

 my $i=0;

 foreach my $class (keys %def_vals) {
     for my $field (keys %{ $def_vals{$class}} ) { 
       $i++;
         $class=uc($class);

#check if the line is the beginning or end of a class
   if ($_ eq $class) {$fclass=$class}
   if ($_ =~ /end$/) {$fclass="";last}

   if ($fclass) {
#if a class was found, split line to get the name of the field and the value
         my @vals=split(" ",$_,2);
         if ($fclass eq $class && $vals[0] eq $field) {
#print "params: $vals[0] , $vals[1]   ----- in $class\n";
#--------------------------------------------
#HERE I PUT SOME CASES WHERE VALUES LOADED FROM FILE ARE MODIFIED BY THE PROGRAM
#FLIPS uses 0 and 1 for not and yes: this simply makes it more readable 
     if ($task =~ /red$/) {
               if ($field =~ /\Ado/) {
                     if (not $vals[1] =~ s/1/yes/) {$vals[1] =~ s/0/no/}
               }
#if making dark, then dark and flat must be set to NO, overscan to YES
               if ($task =~ /\Adark/) {
                     if ($class eq 'DARK') {
                           if ($field eq 'dodark') {$vals[1]='no'} }
		     if ($class eq 'FLAT') {  
                           if ($field eq 'doflat') {$vals[1]='no'}
                       }
                }
#if making flat, then flat must be set to NO, but dark and overscan to yes
               elsif ($task =~ /\Aflat/) {
                     if ($class eq 'FLAT') {
                           if ($field eq 'doflat') {$vals[1]='no'}
                       }
                }
	}	
#if $inlist was given in the arguments and is not blank, then use that value
               if ($class eq "INPUT" && $field eq "name" && $inlist ne "") {
             #in case imcombine is used, then prepend a @ to $inlist
	             if ($task =~ /comb$/) {$vals[1]="\@".$inlist} else {$vals[1]=$inlist}
		} 

# if imcombine, then the output list is what was given as input	
               if ($task =~ /comb$/) {
	          if ($class eq "OUTPUT" && $field eq "name" && $inlist ne "") {$vals[1]=$inlist}
                  elsif ($task =~ /\Adark/ && $class eq "OUTPUT" && $field eq 'rescale') {$vals[1]='0'}
                  elsif ($task =~ /\Aflat/ && $class eq "OUTPUT" && $field eq 'rescale') {$vals[1]='2'}
		  elsif ($task =~ /\Accd/ && $class eq "OUTPUT" && $field eq 'rescale') {$vals[1]='1'}
               } 
#--------------------------------------------
#ok, display values
               $frames[$i]->delete('0','end');
               $frames[$i]->insert('0',$vals[1]);
               last;
          }
    }
  } #end foreach on $field
 }  #end foreach on $class
 
 } #end while	 
 

 close (FILE);

} #end LoadParams


sub SaveParams
  {
my $file = shift;
my $action=shift; 

my $value="";

 open (FILE, ">$file");

 my $pclass="";

my $i=0;
 
 foreach my $class (keys %def_vals) {   
     for my $field  (keys %{ $def_vals{$class}} ) {  
	 $i++;
   if ($main::redmode eq "interactive") {$value=$frames[$i]->get} else {$value=$def_vals{$class}{$field}}


   $class=uc($class);
 
  if ($i == 1) {print FILE "$class\n"}
  elsif ($class ne $pclass) {print FILE "\tend\n$class\n"};
#Substitutes back yes and no with 1 and 0, as required by FLIPS         
   if ($field =~ /\Ado/) {
print "$field $value\n";
	   if (not $value =~ s/no/0/i) {$value =~ s/yes/1/i}
      }
#Copy the list of images into the global $inlist variable
   if ($class eq "INPUT" && $field eq "name") {
           if ($task !~ /comb$/) {$inlist=$value} else {
#In case of IMCOMBINE, input must be a list of lists, instead of the final list........
# First copy it from $outlist
              if ($outlist=="" && $main::redmode eq "interactive") {$outlist=$frames[2]->get}
              $inlist=$outlist;  print "outlist: $outlist\n";
# then  take off the path
                my $nopath=$inlist;
		my @string=reverse(split(/\//,$nopath));
		$nopath=$string[0];
#add a @.....
		$nopath="@".$nopath;
# and put back the path, the result is   <path>/@list ->  <path>/@@list
		$inlist=~ s/$string[0]/$nopath/;
		$value=$inlist;
          # display the new value for clarity
                if ($main::redmode eq "interactive" || ($main::redmode eq "listonly" && $main::debug eq 'y')) {
                      $frames[$i]->delete('0', 'end');
                      $frames[$i]->insert('0', $value);
                   }
         }
      };
   if ($class eq "OUTPUT" && $field eq "name") {
	       if ($task !~ /comb$/) {
#make the  output list	name; same procedure as before with <path>/@list ->  <path>/@out-list
                my $nopath=$inlist;
		my @string=reverse(split(/\//,$nopath));
		$nopath=$string[0];
		$nopath=~ tr/@//d; #we must delete the @
		$nopath="\@out-".$nopath;
	        $outlist=$inlist;
		$outlist=~ s/$string[0]/$nopath/;
		$value=$outlist;
              } else {$value=$outlist}
     # display the new value for clarity
          if ($main::redmode ne "auto") {
            $frames[$i]->delete('0', 'end');
            $frames[$i]->insert('0', $value);
          }
     #Store the output list name into the global variable $outlist       
           $outlist=$value;
	   #create the output list adding an extension (not for imcombine, that's another case;
	    if ($action eq "go" && $task !~ /comb$/)  {&main::addExt($inlist, $value)};
	   }  
   if ($action eq "go") {
      if ($task eq 'objectccdred' && $class eq "FLAT" &&  $field eq "fact") {
            $value=&normalFlat;
             if ($main::redmode ne "auto") {
	       $frames[$i]->delete('0', 'end');
               $frames[$i]->insert('0', $value);
             }
	 }
      if  ($task =~ /comb$/ && $class eq "USER" && $field eq "saturation") {$satur=$value}
   }
        print FILE "\t$field  $value\n";
      $pclass=$class;
   } #end foreach $field
 } #end foreach $class

print FILE "\tend\n";
close (FILE);

 if ($main::redmode ne "interactive") {system("cp $file $tpfile")}

if ($main::redmode eq "interactive" && $action eq "go" &&  $task !~ /comb$/) {
#if not batch mode, check that the input list really exists, unless IMCOMB is used

	        if (not -e $inlist) {
	       		$main::top->Dialog(
                            -title  => 'ERROR !!!',
                            -text   => "Can't find the input list ($inlist),\n please make it",
                            -bitmap => 'error',
                        )->Show;
			&main::mklistAction;
		      }
            }


} #end SaveParams


sub msgHelp
{

     my($topic, $x, $y) = @_;
    if ($topic eq '') {return;};

my $idx=0;
my $i=0;

     my ($sfield, $sentry);

 my $msg;
 foreach my $field (keys %def_vals) {
     for my $entry (keys %{ $def_vals{$field}} ) {   
       $i++;

         if ($topic eq $frames[$idx]) {
            $idx=$i; $sfield=$field; $sentry=$entry; last
         }
    }
 }

if ($idx >0 && defined $flips_help::helpTopics[$idx]) {
   $msg=$flips_help::helpTopics[$idx]} else
   {$msg="Sorry, no help !"}

    $hdlg->Subwidget('message')->configure(
        -text => "Information on $task $def_vals{$sfield}{$sentry}\n\n$msg");
    $hdlg->Show;
} #end sub Help

sub normalFlat
{
# normalize flats
#   $flist=shift;

if (! -e "flatmode.dat") {
   open (OFILE, ">> $main::logfile");
    print OFILE "Warning: could not normalize flat ! \n";
   close (OFILE);
   return (1);
}

   open (MFILE, "flatmode.dat");
   my @modflat=<MFILE>;
   close(MFILE);
 
my $maxval=-999;

  foreach my $modflat (@modflat) {
     chomp($modflat);
      my @md=split(/\t/,$modflat,2);
       if ($md[1] gt $maxval) {$maxval=$md[1]}
  }

#print "max: $maxval\n";

    return $maxval;

} #end normalFlat



#sub normalFlat
#{
## normalize flats
#   $flist=shift;

#   my $ret=1;
#print "$flist : Norm flat \n";
#   open (FILE, "immode.dat") || return (0);
#   my @modflat=<FILE>;
#   close(FILE);
  
#   open (FILE, $flist);
#   while (<FILE>) {
#     chomp($_);
#     my $file=$_;
## Find all entries in immode.dat for the current file (should be only 1....)
#     my @xf=grep(/$file/,@modflat);
#     if (scalar(@xf) != 1) {print "Something wrong with $file in immode.dat\n";return(0)}
#       my @md=split(/\t/,$xf[0],2);
#       if ($md[0] eq $file) {
#           &doImarith($file,"/",$md[1], $file);
#	 }
#      }

#    close (FILE);
#   return $ret;

#} #end normalFlat

sub doImarith
{
# run imarith
 my ($input, $op, $val, $output) =@_;
# $val=chomp($val);

if ($output eq $input) {$output='n'.$input}

  open (IFILE, "> imarith.param");
      print IFILE "INPUT\n\tname $input\n\tend\n";
      print IFILE "OPERATION\n\toperateur $op\n";
      print IFILE "\timage 0\n";
      print IFILE "\tconstant $val\n";
      print IFILE "\tend\n";
      print IFILE "OUTPUT\n\tname $output\n\tend\n";   
  close (IFILE);

my  $result=flipsrun ("arith", "0");
 system ("mv $output $input");
print "$result\n";
} #end doImarith


sub lmod2comb {
#read content of the immode-stat.dat file created by IMMODE.
#reorganize it by CCD number,
#throw away frames with counts higher than some limit    

#Make the input/output lists for imcombine from the output of the immode task
   my $finput=shift;
   my $foutput=shift; 
   my $satur=shift;

 #print "lmod2comb called: it received $finput, $foutput\n";
  
    my $maxccd=20;
#nb: UK and CFHT files have different formats    


#Take the name of the output list (e.g @dark.lst) and keep only the root (e.g. dark)
     my @oroot=split(/\./,$foutput,2);
     $oroot[0]=~ s/\@?//;
#Do same for the input list
        my @string=split(/\./,$finput,2);
	$string[0]=~ s/\@?//;
	# add back the dot only if there was one (eg. yes for @dark.lst, no for @dark)
	if ($string[1] ne "") {$string[1]="\.".$string[1]}
	
     open (FINPUT, "./immode-stat.dat") || return;
     my @all=<FINPUT>;
     close (FINPUT);
    
     open (FOUTPUT, ">$foutput"); 

       open (FOUTLLIST, ">$finput");

#assume that it is CFHT
     my $dflag=1;

   my $dig='';

     foreach my $i (0..$maxccd) {
# add a O before number of CCD   
        if ($i < 10 && $dflag) {$dig='0'}
        else {$dig=''};
# find all files for the same CCD
        #check number of elements found	
        my $nstr=grep(/$dig$i([A-Z]|[a-z])?\.fits/, @all);
        if ($i==0 && $nstr ==0) {
# nothing found: try with UK format (no zero)	
               $dflag=0;
               $dig='';
               $nstr=grep(/$dig$i([A-Z]|[a-z])?\.fits/, @all);
	       #if still nothing, something is wrong (there must be AT LEAST 1 CCD....) !
	         if ($nstr == 0) {print "Can't find any file !!!!!\n";return};
	     }
	 # ok, extract elements    
         my @xstr = grep(/$dig$i([A-Z]|[a-z])?\.fits/, @all); 
# no more CCDs, end loop
	 last if ($nstr==0);

	my $fname=$string[0].$dig.$i.$string[1];

#write to a single LIST the entries from immode-stat.dat with same CCD number
       open (FOUTSLIST, ">$fname");
        foreach my $xstr (@xstr) {
          my @str=split(/\t/, $xstr,4);
          my $mean=$str[2];
           
          {if ($mean < 0)  {print "Found $mean in $xstr, skipping !!!\n"}
          elsif ($satur>0 && $mean > $satur)  {print "Found $mean in $xstr, skipping !!!\n"}
          else{print FOUTSLIST $xstr}}

        }
       close (FOUTSLIST);
 
#append to a file the name of the above list (list of lists)
          print FOUTLLIST "$fname\n";

#now append to another file the name of the combined output frames (one for each CCD)  
       print FOUTPUT $oroot[0].$dig.$i."\.fits\n";

     }        

      close (FOUTLLIST);

     close (FOUTPUT);

} #end lmod2comb



sub build_name {

my $task=shift;



#to access 
# print $def_vals{input}{name}
#  $def_vals{input}{name}= st
# OR foreach  $field ( keys %def_vals ) {
#                for $entry ( keys %{ $def_vals{$field} ) {
#                     .......
#                  }
#             } 


if ($task =~ /red$/) {
 $mainfile="./imred.param";
# using hashes of hashes
  %def_vals= (
     "input" => {
        "name" => ''
       },
     "output" => {
        "name" => '',
        "fulldyn" => '0',
        "comment" => 'Processed by pipeline'
       },    
     "overscan" => {
        "doover" => 'no',
        "mod2d" => '0',
        "ovscan" => '',
    },
     "dark" => {
        "dodark" => 'no',
        "file" => '',
        "darkadu" => '',
        "fact" => '1',
        "name" => ''
     },
     "flat" => {
        "doflat" => 'no',
        "fact" => '',
        "name" => ''
     },
     "mask" => {
        "domask" => 'no',
        "name" => ''
     }
	      );

   $go=$go."IMRED";

  }

elsif ($task =~ /comb$/) {
  $mainfile="./imcombine.param";

  %def_vals= (
     "input" => {
        "name" => ''
          },
     "output" => {
          "name" => '',
          "32bit" => '0',
          "rescale" => '0',
          "mode" => '0',
          "comment" => ''
          },
     "clipping" => {
          "median" => '0',
          "lsigmas" => '0',
          "hsigmas" =>'0',
          "lcut" =>'0',
          "hcut" =>'0'
          }, 
     "ccdclipping" => {
          "ccdclip" => '0',
          "noise" => '0',
          "gain" => '1',
          "e_rqe" => '0'
          },
     "buffer" => {
          "height" => '0'
          },
     "verbose" => {
          "verbose" => '0',
          "x" => '',
          "y" => ''
	  },
     "user" => {
          "saturation" => '0',
          "doclean" => ''
     }
	      );

  $go=$go."IMCOMB";

}

elsif ($task =~ /mode$/ || $task =~ /norm$/) {
  $mainfile="./immode.param";

  %def_vals= (
     "input" => {
        "name" => '',
        "dyn_min" => '0',
        "dyn_max" => '65535'
          },
     "raster" => {
        "xc" => '100',
        "yc" => '200',
        "sizex" => '2000',
        "sizey" => '4000'
	},
     "stat" => {
        "mode" => '0',
        "percent" => '40'
	}
	      );

  $go=$go."IMMODE";
}


$go="Run ";

 #this is the "default" parameter file for each task (bias, dark, flat and process)
  $tpfile=$task.'.param';


@nfields=keys %def_vals; #store here the names of the fields

} #end sub buildName


sub mkConfig {
  my $step=shift;

open (IFILE, "batch.par");
   my @all=<IFILE>;
   chomp(@all);
close (IFILE);

 # extract all  info for the given step from the batch configuration file
  my @allsteps=();
#define which steps should be performed

  if ($step =~ /\Abias/) {@allsteps=("overscan","bias")}
  elsif ($step =~ /\Adark/) {@allsteps=("overscan","bias","dark")}
  elsif ($step =~ /\Aflat/) {@allsteps=("overscan","bias","dark","flat")}
  elsif ($step =~ /\Asflat/) {@allsteps=("overscan","bias","dark","flat","sflat")}
  elsif ($step =~ /\Aobject/) {@allsteps=("overscan","bias","dark","flat","sflat","object")}

  my $dobias="no";
  my $dodark="no";
  my $doflat="no";
  my $dosflat="no";
  my $domask="no";

  my $doos='no';
  my $ossec='';

  my ($nbias, $ndark, $nflat, $nsflat);

#loop on each step
  foreach my $dostep (@allsteps) {

#extract from the configuration array the entries for the current step
# the \A is needed to discriminate between flat and sflat
   my @arr=grep(/\A$dostep/,@all);

#loop on entries
   foreach my $entry (@arr) {
      chomp($entry);
      my @string=split(/ /,$entry,2);
    #this is the value
      my $val=$string[1];
        $val =~ tr/ //d;
       #separate step and field
        @string=split(/:/,$string[0],2);
        if ($dostep eq "overscan") {
            if ($string[1] eq "section") {
	      $_=$val;
              my @n=/(\d{1,4}):(\d{1,4}),(\d{1,4}):(\d{1,4})/;

	      $ossec=$n[0]." ".$n[1]}
            elsif ($string[1] eq "apply") {$doos=$val}
         }  else {
          #for each step, name of output list should be stored in case it is needed later
            if ($dostep eq "bias") {
               if ($string[1] eq "apply") {$dobias=$val}
               elsif ($string[1] eq "output") {$nbias=$val}
	       if ($step =~ /\Abias/) {$dobias="no"} 
             }
             elsif ($dostep eq "dark") {
               if ($string[1] eq "apply") {$dodark=$val}
               elsif ($string[1] eq "output") {$ndark=$val}
	      if ($step =~ /\Adark/) {$dodark="no"} 
             }
             elsif ($dostep eq "flat") {
               if ($string[1] eq "apply") {$doflat=$val}
               elsif ($string[1] eq "output") {$nflat=$val}
	       if ($step =~ /\Aflat/) {
	            $doflat="no";
		    $dosflat="no" }
             }
             elsif ($dostep eq "sflat") {
                 if ($string[1] eq "apply") {$dosflat=$val}
                 elsif ($string[1] eq "output") {$nsflat=$val}
		  if ($step =~ /\Asflat/) {$dosflat="no"}
             }
         #Store input and output lists only for the last step
           if  ($step =~ /\A$dostep/) {
            #assign a value to $inlist only if it is empty
               if ($inlist !~ /\w+/ && $string[1] eq "input") {print "$inlist changed \n";$inlist=$val}
               elsif ($string[1] eq "output") {$outlist=$val}
           }
         }
     }  #end loop
  } #end loop on steps

#imred
  if ($step =~ /red$/) {

      $def_vals{"input"}{"name"}=$inlist;
      $def_vals{"overscan"}{"doover"}=$doos;
      $def_vals{"dark"}{"dodark"}=$dodark;
      $def_vals{"dark"}{"file"}=$ndark;
      $def_vals{"flat"}{"doflat"}=$doflat;
      $def_vals{"flat"}{"file"}=$nflat;
      $def_vals{"mask"}{"domask"}=$domask;

   }

#imcomb
   elsif ($step =~ /comb$/) {

     $def_vals{"output"}{"name"}=$outlist;

 }
#immode - flatnorm is just a call to immode, but it must take the output list as argument
   elsif ($step =~ /mode$/ || $step =~ /norm$/) {

   $def_vals{"input"}{"name"}=$inlist;

    if ( $step =~ /norm$/) {$def_vals{"input"}{"name"}=$outlist}
   }


} #end sub MkConfig








1;



