#!/usr/local/bin/perl

#use English;

use Cwd;
use strict;
#use diagnostics -verbose;


use Tk;
use Tk::Dialog;

use Tk::Table;
use Tk::BrowseEntry;
use Tk::LabFrame;
use Tk::IO;

#use Tools::mklist;     # Create lists of files
use Tools::setdata;		# Create a db file
use Tools::CheckDB;		# Checks the names in the db file
use Tools::MkTree;		# Reorganize data creating links

use File::Basename;

use Common::callPck;		# Execute the proper steps with the given package

use Common::misc;		#miscellaneous functions
use Instr::IConfig;

use GUI::get_params;		# Change the batch-mode parameters
use GUI::displog;
use GUI::FileDialog;

use vars qw/$wconf $NextProc $package  $redmode $debug $top/;

use vars qw/$msgid $mpid $dpid $bpid $procInd $pIndWdw $toolWdw $AdmWdw/;

use vars qw/$RunConf $handle/;

#/////////////////////////
# initialization


&Init();			#define these variables, the sub is in Common::misc.pm
&LoadPrefs();


#/////////////////////////
# some initializations
$package="PDL";
$redmode="RUN";
$bpid=-1;

#\\\\\\\\\\\\\\\\\\\\\\\\\

# Enable trapping of ctrl-C (from man perlipc)

sub catch_zap {
  my $signame = shift;
  warn "I (the GUI) got a CTRL-C: killing $bpid !"; 	  
  if ($bpid>0) {
warn "#DEBUG killing $bpid" if ($DEBUG);
     kill ("KILL", $bpid) if ($bpid>0); #killing the batch process
     displog::check('abort');
   } else {
      &Logout
   }
}

$SIG{INT} = \&catch_zap;	# best strategy

sub getmsg {
  my $l;
  while ($l=$handle->readline)  {

    if ($l =~ /^\#DEBUG/) {warn "$l"}
    else {displog::check($l)}
  }
  $handle->close; 
  return; 
}


#------------------------------------------
# Phase 0: create the front end.
#------------------------------------------


my $title=uc($main::SOFT->{NAME}[1]);
$top = MainWindow->new(-name => 'Prered', -title=>"$title",);
my $lf;

my $frame = $top->Frame(-relief => 'flat');
$frame->pack(-side=>'top');
my @pl = qw/-side top -expand 1 -padx .1c -pady .1c/;


my $buttons = $top->Frame();
$buttons->pack(-side => 'top', -pady => '2');


# HANDLING OF THE PIPELINE

#Draw the header
my $lfr = $top-> Frame();
$lfr->pack(-side=>'top',-anchor=>'w', -expand=>1);

my @steps=('Bias', 'Dark','DFlat', 'Tflat', 'Fringe', 'Sflat', 'Science','  STOP AT: ');

my $c=0;
foreach my $step (@steps) {
  my $e_step=$lfr->Label(-text=>$step, -relief=>'groove', -width=>8);
  $e_step->grid(-row=>1, -column=>$c++);
}

#Draw the configuration stuff  (checkboxes, buttons)

$wconf=$top->Scrolled(qw/Text -width 80 -height 2 -width 105 -wrap none -cursor
		      top_left_arrow  -scrollbars se/);
$wconf->pack(qw/-side top -anchor w -expand 1 -fill both/);

AddIConfig();


#Draw the commands
$lfr = $top-> Frame();
$lfr->pack(-anchor=>'w', -expand=>1);

#my $b_leg = $lfr->Button(-text => 'Help');
#my $plegWdw;
#$b_leg->configure(-command => (sub{
#				 if (! Exists($plegWdw)) {
#				   $plegWdw= $main::top->Toplevel;
#				 }
#			       }));

my $b_more = $lfr->Button(-text => 'Add a configuration', -command => \&AddIConfig);

my $b_go = $lfr->Button(-text => 'Go',
			-cursor=>'shuttle');

$b_go->configure(-command=>(sub {
			      my $nprocs=IConfig->Count();        
			      my $message;
			      if ($nprocs >0) {
				$message="Please confirm that you want to RUN the pipeline (processes: $nprocs, mode: $redmode).";
			      } else {$message="No process to perform !\n";}
			      my $button = $top->messageBox('-icon' => 'question', -type => 'OkCancel',
							    -title => 'Message',
							    -message => $message);
			      if ($button eq 'Ok') {
				$b_go->configure(-state=>'disabled');              
				
				&RunPRERED;
				$b_go->configure(-state=>'normal');
				
			      }
			    }
			   ) );


my $b_update=$lfr->Button(-text => 'Update Window', -command=>(sub {ResetForm()}) );

#This allows to kill everything from here !

my $b_stop=$lfr->Button(-text => 'STOP', -cursor=>'pirate', -command=>(sub { return if (!$RunConf); 
									 my $button = $top->messageBox('-icon' => 'question', -type => 'YesNo',
												       -title => 'WARNING',
												       -message => 'Are you REALLY sure that you want to STOP the pipeline ?');
									 if ($button eq 'Yes') {
									   my $ppid=$$;	#Killing
									   kill ("INT", $ppid);
									 }
								       }
								      ) );

$b_update->grid($b_more);

$b_go->grid($b_stop);


#------------------------------------------
# Phase 1: Add menus and accelerators
#------------------------------------------

my $menu = $top->Frame(-relief => 'raised', -borderwidth => '1');
$menu->pack(-before => $frame, -side => 'top', -fill => 'x');

my $menu_file = $menu->Menubutton(-text => 'File', -underline => 0);

$menu_file->command(-label => 'Exit', -command => sub{&Logout}, -underline => 0);
$menu_file->pack(-side => 'left');


my $menu_file_m = $menu_file->cget(-menu);
$menu_file_m->entryconfigure(2, -accelerator => 'Ctrl+Q');
$top->bind('<Control-q>' => sub{&Logout()});

my $menu_pack=$menu->Menubutton(-text => 'Package', -underline => 0)->pack(-side => 'left');


foreach (qw/FLIPS PDL/)  {
  $menu_pack->radiobutton(-label=>$_, -variable=> \$package, -value => $_)
}

my $menu_mode=$menu->Menubutton(-text => 'Mode', -underline => 0)->pack(-side => 'left');

foreach (qw/listonly mklists RUN/)  {
  $menu_mode->radiobutton(-label=>$_, -variable=> \$redmode, -value => $_)
}

my $menu_tools=$menu->Menubutton(-text => 'Tools',
				 -menuitems => [['command' => 'Administration',
						 -command => \&AdmAction],
						['command' => 'Tools',
						 -command => \&toolsAction]]);

$menu_tools->pack(-side => 'left');


my $menu_help=$menu->Menubutton(-text => 'Help',
				 -menuitems => [['command' => 'About Prerepix',
						 -command => sub {} ],
						['command' => 'Flags',
						 -command =>sub {} ]]);

$menu_help->pack(-side => 'right');

displog::Display ();

#------------------------------------------
# All done, we can start !
#------------------------------------------


MainLoop;

# end of main program



sub ResetForm
  {
    while (my $conf=IConfig->each)   {
      UpdateForm($conf);
    }
  }				#end ResetForm


sub UpdateForm
  # Purpose: #check the status of reduction and 
  #  assign a color to the checkboxes accordingly
  {
    
    my $conf=shift;
    
    my @steps=$conf->getSteps;
    foreach my $step (@steps) {

      my $check=$conf->CheckBox($step);
      #first check if output files exist already
      my ($ncheck, $ntot)= $conf->checkList($step, 'output') ;

      #if ($ncheck >0 && ($ncheck % $conf->{NUMCCD} ==0) ) { 
      if ($ncheck==$ntot) {
        $check->configure(-selectcolor=>'green');
        $conf->Done($step, 1);
      }
      elsif ($ncheck>=1) {$check->configure(-selectcolor=>'yellow')}
      else {
	#no output file.... so check if the input list exists
        #$list= $conf->getList($step, 'input');
        ($ncheck, $ntot)= $conf->checkList($step, 'input') ;
        if ($ncheck==-1) {	#the INPUT list was not found
	  $check->configure(-selectcolor=>'red');
	}
        elsif ($ncheck==0) {	#the files in the INPUT list do not exist
	  $check->configure(-selectcolor=>'brown');
	}
	elsif ($ncheck>=1) {	#ok
	  $check->configure(-selectcolor=>'blue');
	}
	
      }
      
    }				#end loop on steps
    
  }				# end UpdateForm


sub AddIConfig
  {
    
    my $newconf=IConfig->mkName;
    my $conf=IConfig->new($newconf);
    
    #my $newconf=$conf->{NAME}; 
    if (-e $newconf) {
      $conf->LoadParams();  
    } else { $conf->Instrument($INSTRUMENT)}  

         
   DrawIConfig(\$conf);

  } #end AddIConfig

sub DrawIConfig
{

my $rconf=shift;
my $conf=$$rconf;

    my $mbutt=$wconf->Menubutton(-text=> "Modify", -relief=>'groove',
				 -menuitems=> [
					       [ 'command'=>'Edit',
						 -command=> sub { 	
						   &get_params_simple::mkform($conf);
						   UpdateForm($conf);
						 }],
					       
					       [ 'command'=>'Save',
						 -command=> sub { 	
						   $conf->SaveParams;
						   UpdateForm($conf);
						 }],
					       [ 'command'=>'Load',
						 -command=> sub {
						   if (-e $conf->{NAME}) {
						     $conf->LoadParams();  
						   }
						 }],
					       
					       [ 'command'=>'Remove',
						 -command=> sub {
						   #my $idx=$conf->getIdx; 
						   my $ok=RmIConfig($conf);
						   undef $conf if ($ok);  
						 }],
					       
					      ]); 
    
    
    my $e_bfile = $wconf->Entry(-width => '15', -relief => 'sunken',
				-textvariable => \$conf->{NAME});
    
    
     my @steps=('bias', 'dark','dflat', 'tflat', 'fringe', 'sflat', 'science', 'skip');
    #my @steps=@{IConfig->lsteps};
    #push @steps, 'skip';
 
    my $e_steps = $wconf->BrowseEntry(-choices=>\@steps, -state=>'readonly', -width =>'10',
				      -variable => \$conf->{TOSTEP});
   
    $conf->Entry($e_steps, $e_bfile, $mbutt); #Store the entries for later use
    
    
    my $ch_bias = $wconf->Checkbutton(
				      # -command=>( sub{ Disable( 'bias', $k) } ),
				      -width=>5,
				      -onvalue => 'yes',
				      -offvalue => 'no',
				      -variable => \$conf->{bias}{apply}{Value},
				      -relief   => 'flat',
				     );
    
    $conf->CheckBox('bias',$ch_bias);	
    
    
    
    my $ch_dark = $wconf->Checkbutton(
				      #-text     => 'calc DARK',
				      # -command=>( sub{ Disable( 'dark' ) } ),
				      -width=>5,
				      -onvalue => 'yes',
				      -offvalue => 'no',
				      -variable => \$conf->{dark}{apply}{Value},
				      -relief   => 'flat');
    
    $conf->CheckBox('dark',$ch_dark);		
    
    
    
    my $ch_dflat = $wconf->Checkbutton(
				       # -text     => 'calc DFLAT',
				       # -command=>(sub{Disable('flat')}),
				       -variable => \$conf->{dflat}{apply}{Value},
				       -width=>6,
				       -onvalue => 'yes',
				       -offvalue => 'no',
				       -relief   => 'flat',
				       -selectcolor=>'blue');
    
    $conf->CheckBox('dflat',$ch_dflat);	
    
    
    
    my $ch_tflat = $wconf->Checkbutton(
				       # -text     => 'calc TFLAT',
				       # -command=>(sub{Disable('flat')}),
				       -variable => \$conf->{tflat}{apply}{Value},
				       -width=>6,
				       -onvalue => 'yes',
				       -offvalue => 'no',
				       -relief   => 'flat',
				       -selectcolor=>'blue');
    
    $conf->CheckBox('tflat',$ch_tflat);	
    
    
    my $ch_sflat = $wconf->Checkbutton(
				       #-command=>(sub{Disable('sflat')}),
				       -variable => \$conf->{sflat}{apply}{Value},
				       -width=>5,
				       -onvalue => 'yes',
				       -offvalue => 'no',
				       -relief   => 'flat',
				       -selectcolor=>'blue');
    
    $conf->CheckBox('sflat',$ch_sflat);	
    
    
    my $ch_frg = $wconf->Checkbutton(
				     # -text     => 'calc FRINGE',
				     # -command=>(sub{Disable('fringe')}),
				     -variable => \$conf->{fringe}{apply}{Value},
				     -width=>5,
				     -onvalue => 'yes',
				     -offvalue => 'no',
				     -relief   => 'flat',
				     -selectcolor=>'blue');
    
    $conf->CheckBox('fringe',$ch_frg);	
    
    
    my $ch_sci = $wconf->Checkbutton(
				     # -text     => 'calc SCIENCE',
				     #-command=>(sub{Disable('science')}),
				     -variable => \$conf->{science}{apply}{Value},
				     -width=>3,
				     -onvalue => 'yes',
				     -offvalue => 'no',
				     -relief   => 'flat',
				     -selectcolor=>'blue');
    
    $conf->CheckBox('science',$ch_sci);		


      foreach my $step (@{$conf->{steps}})  {
	my $ref=$conf->CheckBox($step);
         next if ($ref == -1);
	 $wconf->windowCreate('end', -window=>$ref);    
      } 

      my @entries=$conf->Entry;
      foreach my $entry (@entries) {
         $wconf->windowCreate('end', -window=>$entry);       
       }
      
    $wconf->configure(-state=>'normal');
    $wconf->insert('end', "\n");
    $wconf->configure(-state=>'disabled');
    
    &UpdateForm($conf);

    
  }				#end sub AddIConfig

sub RmIConfig
  {
    my $rconf=shift;
  

     while (my $conf = IConfig->each) { 
      my @entries=$conf->Entry;
      foreach my $entry (@entries) {
          $entry->destroy if ($entry ne '' && Exists($entry));       
       }
      
      foreach my $step (@{$conf->{steps}})  {
	my $ref=$conf->CheckBox($step);
	   $ref->destroy if ($ref !=-1);
      } 
    }

     
    my $index=$rconf->Remove; #remove the configuration and give its position
   
#Now delete and redraw all      
      $wconf->configure(-state=>'normal');



      $wconf->delete('1.0','end');

      while (my $conf = IConfig->each) {    
          DrawIConfig(\$conf);
          $wconf->insert('end', "\n");
      }
      
      
      $wconf->configure(-state=>'disabled');
      
      $index=1;
          
    
    return $index;
    
  }				#end sub rmconfig


sub DelList {
  
  my $list="";
  if ($#_ >-1) {$list=shift}
  
  if ($list eq "") {
    &getFile('load',\$list, [qw/.lst .list/], 'Select the list of files to be deleted');
  }

  
  if ($list eq "") {return}  
  
  my $msg=sprintf "All the files in the list %s will be removed, are you sure ?", $list;
  
  my $button = $top->messageBox('-icon' => 'warning', -type => 'OKCancel',
				-title => 'WARNING',
				-message => $msg);
  
  
  if ($button eq 'Ok') {
    my $ndel=&AskCleanup($list);
     $msg=sprintf "%d files removed....", $ndel;
     $button = $top->messageBox('-icon' => 'warning', -type => 'OK',
				-title => 'WARNING',
				-message => $msg);
  }
  
}				#end sub DelList




sub getOffAction {
  
  my $target="";
  &getFile('load',\$target, '.coo','Select the TARGET file');
  if ($target eq "") {return}
  my $reference="";
  &getFile('load',\$reference, '.coo','Select the REFERENCE file');
  if ($reference eq "") {return}
  
  require  Tools::getOffset;	# Compute the offset between frames
  &calcoff($target, $reference);
  
}

sub par_checkDB {
  my $tol=15;
  my $idb='setdata.db';
  my $odb='setdata_n.db';
  
  
  my $chdbWdw= $main::top->Toplevel;
  $chdbWdw->title('Check database');
  
  my $buttons = $chdbWdw->Frame();
  $buttons->pack(-side => 'bottom', -pady => '2', -anchor => 'center');
  
  my $buttons_fromdb = $buttons->Button(-text => 'Input DB');
  my $buttons_todb = $buttons->Button(-text => 'Output DB');
  $buttons_fromdb->pack(-side => 'top', -pady => '10');
  $buttons_todb->pack(-side => 'top', -pady => '10');
  
  $buttons_fromdb->configure(-command => (sub { &getFile('load',\$idb, '.db','Select the INPUT file')} ));
  $buttons_todb->configure(-command => (sub { &getFile('load',\$odb, '.db','Select the OUTPUT file')} ));
  
  my $buttons_OK = $buttons->Button(-text => 'OK');
  $buttons_OK->pack(-side => 'top', -pady => '10');
  $buttons_OK->configure(-command => ( sub {
					 system ("perl $BIN/../Tools/CheckDB.pl -i $idb -o $odb -t $tol&")    
				       } ));
  
  
  my $buttons_close = $buttons->Button(-text => 'Close');
  $buttons_close->pack(-side => 'top', -pady => '10');
  $buttons_close->configure(-command => (sub {$chdbWdw->destroy}) );
  
  
  my $f = $chdbWdw->LabFrame(-label => "Settings", -labelside => "acrosstop");
  $f->LabEntry(-label => "Tolerance (arcmin)", -textvariable => \$tol)->pack;
  $f->LabEntry(-label => "Input DB", -textvariable => \$idb)->pack;
  $f->LabEntry(-label => "Output DB", -textvariable => \$odb)->pack;
  $f->pack;
  
}

sub AdmAction {
  
  my $c;
  
  if (! Exists($AdmWdw)) {
    $AdmWdw= $main::top->Toplevel;
    $AdmWdw->title('Administration');
    
    my $buttons = $AdmWdw->Frame();
 
    #=== External binaries (Swarp, WW)

    my $lf = $buttons->LabFrame(-label => "Binaries", -labelside => "acrosstop");
    my @pl = qw/-side left -expand 1 -padx .1c -pady .1c/;
    $c = $lf->Frame->pack(@pl);

    $c->Label(-text => "Sextractor: ", -anchor => 'e')->pack;
    my $eSEx=$c->Entry(-width => '25', -relief => 'sunken',
			      -textvariable => \$SExbin)->pack;
    $c->Label(-text => "WeightWatcher: ", -anchor => 'e')->pack;
    my $eWW=$c->Entry(-width => '25', -relief => 'sunken',
			      -textvariable => \$WWbin)->pack;
    $lf->pack;
    

    
    #The working directory
    $buttons->Label(-text => "Current dir: ", -anchor => 'e')->pack(-side => 'top');
    my $e_wdir=$buttons->Entry(-width => '25', -relief => 'sunken',
			       -textvariable => \$WORK_DIR)->pack(-side=> 'top');
    
    # The instrument to be used
    $c = $buttons->BrowseEntry(-choices=>$IMPL_INSTR, -label => "Instrument:", 
			       -variable => \$INSTRUMENT, -state=>'readonly');
    $c->pack(-pady=>'5');
    #    foreach (keys (%INSTR_TAB)) {
    #      $c->insert("end", $_);
    #    }
    
    #maximum amount of memory
    $buttons->Label(-text => "Max. memory to combine images (Mb) : ", -anchor => 'e')->pack(-side => 'top');
    my $e_mem=$buttons->Entry(-width => '5', -relief => 'sunken',
			      -textvariable => \$MAXMEM)->pack(-side=> 'top');
    
    
    
    #Redo steps ?
    # $c = $buttons->BrowseEntry(-label => "Redo: ", -variable => \$REDO, -state=>'readonly');
    # $c->pack(-pady=>'5');
    # $c->insert("end", "yes");
    # $c->insert("end", "no");
    my $lf = $buttons->LabFrame(-label => "REDO steps", -labelside => "acrosstop");
    my @pl = qw/-side left -expand 1 -padx .1c -pady .1c/;
    $c = $lf->Frame->pack(@pl);
    
    $c->Radiobutton(
		    -command => (sub{
				   my $button = $top->messageBox('-icon' => 'warning', -type => 'Ok',
								 -title => 'WARNING',
								 -message => 'All existing images for the selected steps will be erased and done again !');
				 }),
		    -text     => "Yes",
		    -variable => \$REDO,
		    -relief   => 'flat',
		    -value    => "yes",
		   )->pack(@pl);
    
    $c->Radiobutton(
		    -command => (sub{
				   my $button = $top->messageBox('-icon' => 'warning', -type => 'Ok',
								 -title => 'WARNING',
								 -message => 'Images will NOT be reprocessed if already existing !'
								);
				 }),
		    -text     => "No",
		    -variable => \$REDO,
		    -relief   => 'flat',
		    -value    => "no",
		   )->pack(@pl);
    $lf->pack;
    
    #Clean temporary results ?
    #$c = $buttons->BrowseEntry(-label => "Clean: ", -variable => \$CLEAN, -state=>'readonly');
    #$c->pack(-pady=>'5');
    #$c->insert("end", "yes");
    #$c->insert("end", "no");
    $lf = $buttons->LabFrame(-label => "CLEAN temporary results", -labelside => "acrosstop");
    @pl = qw/-side left -expand 1 -padx .1c -pady .1c/;
    $c = $lf->Frame->pack(@pl);
    
    $c->Radiobutton(
		    -command => (sub{
	 my $button = $top->messageBox('-icon' => 'warning', -type => 'Ok',
	  -title => 'WARNING',
	 -message => 'Only calibration (bias/dark/dflat/tflat) TEMPORARY images will be erased !');
				 }),
		    -text     => "Remove calib",
		    -variable => \$CLEAN,
		    -relief   => 'flat',
		    -value    => "calib",
		   )->pack(@pl);


   $c->Radiobutton(
		   -command => (sub{
	  my $button = $top->messageBox('-icon' => 'warning', -type => 'Ok',
	   -title => 'WARNING',
	   -message => 'All TEMPORARY images created during each step will be erased !');
		   }),
		   -text     => "Remove ALL",
		   -variable => \$CLEAN,
		   -relief   => 'flat',
		   -value    => "all",
		  )->pack(@pl);
    
    
    
    $c->Radiobutton(
		   -command => (sub{
	  my $button = $top->messageBox('-icon' => 'warning', -type => 'Ok',
	   -title => 'WARNING',
	   -message => 'All images will be kept on disk');
		   }),
		    -text     => "Remove none",
		    -variable => \$CLEAN,
		    -relief   => 'flat',
		    -value    => "no",
		   )->pack(@pl);
    $lf->pack;
    

    $buttons->Checkbutton(
		   -command => (sub{
	  my $button = $top->messageBox('-icon' => 'warning', -type => 'Ok',
	   -title => 'WARNING',
	   -message => 'This will produce a lot of infos.... used for debugging');
		   }),
		    -text     => "VERBOSE",
		    -variable => \$DEBUG,
		    -relief   => 'flat',
		   )->pack(-side=>'top');

    my $buttons_undo = $buttons->Button(-text => 'Undo changes');
    $buttons_undo->pack(-side => 'left', -padx => '10', -pady => '10');
    $buttons_undo->configure(-command => (sub {&LoadPrefs()}) );

    my $buttons_save = $buttons->Button(-text => 'Save');
    $buttons_save->pack(-side => 'left', -padx => '10', -pady => '10');
    $buttons_save->configure(-command => (sub {&SavePrefs()}) );
    
    my $buttons_close = $buttons->Button(-text => 'Quit');
    $buttons_close->pack(-side => 'bottom', -padx => '10', -pady => '10');
    $buttons_close->configure(-command => (sub {$AdmWdw->destroy}) );
    
    $buttons->pack;
    
    
  }
  
}



sub toolsAction {
  if (! Exists($toolWdw)) {
    $toolWdw= $main::top->Toplevel;
    $toolWdw->title('Tools');
    
    my $buttons = $toolWdw->Frame();
    $buttons->pack(-side => 'bottom', -pady => '2', -anchor => 'center');
    
    #my $buttons_setdata = $buttons->Button(-text => 'Build DB');
    my $buttons_checkdb = $buttons->Button(-text => 'Check DB');
    my $buttons_mklinks = $buttons->Button(-text => 'Make Links');
    my $buttons_mklist = $buttons->Button(-text => 'Build Lists');
    my $buttons_dellist = $buttons->Button(-text => 'Delete Lists');
    #my $buttons_getOffset = $buttons->Button(-text => 'Calc Offset');
    
    #$buttons_setdata->pack(-side => 'top', -pady => '2');
    $buttons_checkdb->pack(-side => 'top', -pady => '10');
    $buttons_mklinks->pack(-side => 'top', -pady => '10');
    $buttons_mklist->pack(-side => 'top', -pady => '10');
    $buttons_dellist->pack(-side => 'top', -pady => '10');
    #$buttons_getOffset->pack(-side => 'top', -pady => '10');
    
    my $buttons_close = $buttons->Button(-text => 'Close');
    $buttons_close->pack(-side => 'top', -pady => '10');
    $buttons_close->configure(-command => (sub {$toolWdw->destroy}) );
    
    
    
    #$buttons_setdata->configure(-command => \&setdata);
    $buttons_checkdb->configure(-command => \&par_checkDB);
    $buttons_mklinks->configure(-command => \&MakeLinks);
    $buttons_mklist->configure(-command =>  (sub {
					       system ("perl $BIN/../Tools/mklist.pl &")
					     })
			      );
    $buttons_dellist->configure(-command => \&DelList);
    #$buttons_getOffset->configure(-command => \&getOffAction);
    
    
  }
  
}				#end sub toolsAction

sub RunPRERED
  {
    
    my $cproc=0;
    
    my @entries=();
    my $nconf=-1;
    
    while (my $conf = IConfig->each) {    
      push @entries, $conf->Entry;
      $nconf++;
    }
    
#Disable some widgets that should not be used during processing
    foreach my $entry (@entries) {$entry->configure(-state=>'disabled') }
   
  Proc: while (my $conf = IConfig->each) {
      $RunConf=$conf;
      $conf->SaveParams;
      my @steps=$conf->getSteps();
      
      next Proc if ($#steps==-1);
      my $parfile=$conf->Name();

      my $cmd="perl $BIN/../batch.pl -gui";
      foreach my $step (@steps) {
        #next if ($REDO eq 'no' && $conf->{$step}{done} ==1);
	$cmd=$cmd." -step $step ";
      }
      $cmd=$cmd." -mode $redmode -c $parfile";
      $bpid=-1;
      
      $handle = Tk::IO->new(-linecommand=>(sub {getmsg}));
print "Prerepix called with: $cmd\n";
       $handle->exec("$cmd");
      $bpid=$handle->pid();
      
      
      undef $NextProc;
      if ($DEBUG) {print ">>>>>>>>> Waiting for NextProc....."; }
      
      $top->waitVariable(\$NextProc);

      if ($NextProc==-1) {IConfig->Reset(); last Proc};  #Something wrong.... stop the processing

      if ($DEBUG) {print " DONE\n";}
      UpdateForm($conf);
      
      if ($redmode eq 'listonly' && $cproc < $nconf)  {
	my $message="Should I proceed to next process ?";
	my $button = $top->messageBox('-icon' => 'question', -type => 'OkCancel',
				      -title => 'Question',
				      -message => $message);
	if ($button eq 'Cancel') {last Proc}
      }	
      $cproc++;
      
    }
    



#Release the widgets    
    foreach my $entry (@entries) { $entry->configure(-state=>'normal') }
    
    $RunConf=0;
    $bpid=-1;

} #end sub RunPrered




sub Logout
{
  print "Exiting..... \n";
  #system ("echo quit >> $main::PIPEFILE");
  my $ret;
  #print "killing $bpid\n";
  kill ("INT", $bpid) if ($bpid>0);
  exit;
}
