package ESO;

use Instr::Header;
@ISA = ("Header");

sub init_head  {
my $self=shift;

#$self->{LAT}=-29.2567;

$self->{_table} =
{
 ORIGIN => 'ORIGIN',
 DETECTOR  => 'INSTRUME',
 RA => 'RA',
 DEC => 'DEC',
 OBSTYPE => 'ESO DPR CATG',  #to be checked......
 EXPTIME => 'EXPTIME',
 FILTER => 'ESO INS FILT NAME',
 DATE => 'DATE-OBS',
 LOCTIME => '',   #it is computed by UT2Local()
 'LST-OBS' => 'LST',
 OBJECT  => 'OBJECT',
 IMAGEID  => '', #it is ESO CHIP[n] INDEX
 FILENAME => 'ORIGFILE',
 # AM => ''        #it is computed by getairmass, requires LST, RA, DEC, LAT
 }

} #end sub init_head


sub Translate
{
 my $self=shift;
 my $file=shift;

  my %head=$self->getVals($file);



#   if ($self->{FILENAME} eq 'UNDEFINED') {
#        my @s=split(/\//, $file );
#	$self->{FILENAME}=$s[-1];
#    }

# ================================================================
#   Last but not least, 'fix' the values to a standard convention
# ================================================================

my $locdate="";

 while  (my ($key, $value) = each %{$self}) {

#warn "####### $key - $value", ref($value);
     next if (ref($value) eq 'HASH' || $key eq 'INAME');
     if ($value) {$value=~tr/\'| //d}

   if  ($key eq 'LST-OBS') {
       #The sidereal time is written in decimal !
         my @lstime=();
         $lstime[0]=int($value/3600);
         $value-=($lstime[0]*3600);
         $lstime[1]=int($value/60);
         $value-=($lstime[1]*60);
         $lstime[2]=int($value+.5);
	 $value=sprintf ("%02d:%02d:%02d",@lstime);
     }
  elsif  ($key eq 'LOCTIME') {
#ESO headers do not have local time, so compute it
    my $date=$self->{DATE};
       $date=~tr/\'| //d;
    my ($utdate,$HHMMSS)=split(/T/,$date,2);
#warn ">>>>>> DATE LOCTIME :  $utdate- $HHMMSS";
    ($locdate, $value)=$self->UT2Local($utdate, $HHMMSS, 4); 
         
#warn "LOCDATE LOCTIME set to $ldate $value";
  }
     elsif  ($key eq 'RA') {
        $value=$self->sexRA($value);
     }
     elsif  ($key eq 'DEC') {
        $value=$self->sexDEC($value);
     } elsif  ($key eq 'FILTER') {
        $value=substr($value,0,4); #take a max of 4 chars
     } 
      $value =~ s/[^a-zA-Z0-9+-:.]+//; #remove all non ascii characters
     $self->{$key}=$value;
  }

#warn "############## ",  $self->{LOCTIME}, $self->{LOCDATE};

if (exists $self->{LOCTIME} && $locdate ne '') {
   $self->{LOCDATE}=$locdate;
   $self->set_locdate;   
}

#if (exists $self->{RA} && exists $self->{DEC} && 
# exists $self->{'LST-OBS'} && exists $self->{LAT}) {
#  $self->setAirMass;
#}


# end fixvalues



}  #end sub Translate



sub setWCS
  {
    my $self=shift; 
    my $head=shift; 
    if (exists $head->{CDELT1} && !exists $head->{CD1_1}) {
      $self->{CD1_1}=$head->{CDELT1};
    }
   if (exists $head->{CDELT2} && !exists $head->{CD2_2}) {
      $self->{CD2_2}=$head->{CDELT2};
    }
    if (!exists $head->{CD1_2}) {$self->{CD1_2} =0.}
    if (!exists $head->{CD2_1}) {$self->{CD2_1} =0.}
  }

sub setCCD 
  {    
    my $self=shift; 
    my $head=shift; 
    
    while (my ($key,$value) = each %$head) {

      if ($key =~ /CHIP\s* INDEX/ || $key =~ /CHIP\d+ INDEX/ || $key eq 'CHIPINDE') {$self->{IMAGEID}=$value; last}
  }
  
}

sub HowMany
#return the number of extensions 
{
my $self=shift;
my $name=shift;
my $numccd=$self->getProps('NUMCCD');

my $nfound=0;

for my $i (1..$numccd) {
 my $ext=$name."[$i]";
 my %h=&fitsio::rhead($ext);
 if (%h) {$nfound++}
}

return $nfound;
}


 1;
