#!/usr/local/bin/perl

package get_params_iraf;


#use Tk;
#use Tk::Dialog;
#use Tk::Table;
#use FindBin;

use Common::FileDialog;
use Iraf::help;
use Tie::IxHash;

use Common::misc;

use strict;

use vars qw/$task $inlist $outlist $proc $go $mainfile $fname @frames $hdlg/;
use vars qw/@nfields %def_vals/;
use vars qw/$fringes $dofringe/;
use vars qw/$scalmod $combmod/;

tie (%def_vals, 'Tie::IxHash');


#------------------------------------------
# Phase 0: create the front end.
#------------------------------------------

sub mkform {

$task=shift;
$inlist=shift;
my $routlist=shift;

#return valued of the reduction = number of errors fiund (0: OK, >=1: something wrong)
my $ret=0;

#The usage of $outlist depends on the calling task
# IF ccdproc, then it may contain the list of ME files, if they are created (OUTPUT to main procedure);
# otherwise, it contains the name of the FINAL list (INPUT from main procedure)
$outlist=$$routlist;


# define the parameters' names and default values
 $proc=&build_name($task);



 $go=$go.uc($proc);


#this is the "default" parameter file for each task (bias, dark, flat and process)
  $mainfile=$task.'.cl';

#IRAF allows to use any name for the script, but this may be better to prepare the automatic mode
 my $defscript=$mainfile;

 if ($main::redmode ne "interactive") {mkConfig($task)}

my $app = "";

#The form for each task is only displayed in interactive mode, or in test mode if the debug flag is set
if ($main::interface eq 'gui' && ($main::redmode eq "interactive" || ($main::redmode eq "listonly" && $main::debug eq 'y') ) ) {

#&iraf_help::defHelp($task);
$app = $main::top->Toplevel;
$app->title('IRAF interface: '.$task);

# Use a scrollable table (useful for task with many parameters)
my $t  = $app->Table(-scrollbars => 'e',-fixedrows => 1, -rows => 20);
$t->pack(-expand=> 1, -fill => 'both');


my $c=0;

$t->put(0,$c++,$t->Label(-text=>"Entry",-anchor=>'w'),-relief => 'raised');
$t->put(0,$c++,$t->Label(-text=>"Value",-anchor=>'w'),-relief => 'raised');

$t->configure(-columns => $c) if ($c > $t->cget('-columns'));



@frames=();
#$nfields=scalar(@names);

$fname=$mainfile;

my $r=1;

foreach my $i (0..$#nfields) {
    # Use symbolic/soft references to create the widget names at object time.
    
     my $c=0;
     my $e;

    ${$e} = $t->Entry(-width => '50', -relief => 'sunken');

    $frames[$r]=${$e};

    $t->put($r,$c++,$t->Label(-text => "$def_vals{$nfields[$i]}->{Name}", -anchor => 'e'));
    $t->put($r,$c++,${$e});

    $r++;

}


my $buttons = $app->Frame();
$buttons->pack(-side => 'bottom', -pady => '2', -anchor => 'center');
my $buttons_clear = $buttons->Button(-text => 'Clear');
my $buttons_load = $buttons->Button(-text => 'Load');
my $buttons_save = $buttons->Button(-text => 'Save as');
my $buttons_savedef = $buttons->Button(-text => 'Save def.');
my $buttons_go = $buttons->Button(-text => $go);
my $buttons_skip = $buttons->Button(-text => 'Next');

#If not interactive mode, windows are only used to show the parameters (read only)
if ($main::redmode eq "interactive") {
   $buttons_clear->pack(-side => 'left', -padx => '2');
   $buttons_load->pack(-side => 'left', -padx => '2');
   $buttons_save->pack(-side => 'left', -padx => '2');
   $buttons_savedef->pack(-side => 'left', -padx => '2');
   $buttons_go->pack(-side => 'left', -padx => '2');
   $buttons_skip->pack(-side => 'left', -padx => '2');
}

#------------------------------------------
# Phase 1: Add menus, dialog boxes
#------------------------------------------


my $menu = $app->Frame(-relief => 'raised', -borderwidth => '1');
$menu->pack(-before => $t, side => 'top', -fill => 'x');

my $menu_file = $menu->Menubutton(-text => 'File', -underline => 0);

$menu_file->command(-label => 'Exit', -command => sub{$ret=1;$app->destroy}, -underline => 0);
$menu_file->pack(-side => 'left');

# ------------------------------------------
# Phase 2: Help section
#------------------------------------------

$hdlg = $app->Dialog(-title => 'Help',-justify => 'left');


$app->bind('<Any-F12>' => sub {
    my $w = shift;
    my $e = $w->XEvent;
    &msgHelp($w->containing($e->X, $e->Y), $e->X, $e->Y);
});

#------------------------------------------
# Phase 4: Miscellaneous other actions
#------------------------------------------


$buttons_clear->configure(-command => \&clearAction);


$buttons_go->configure(-command => \&goAction);


$buttons_load->configure(-command => sub { &fileAction('load')} );

$buttons_save->configure(-command => sub{ &fileAction('save')});

#This is needed to prepare the scripts for the automatic mode,
# when the scripts are created using the default names
$buttons_savedef->configure(-command => sub{&SaveParams($defscript)});

$buttons_skip->configure(-command => sub{$app->destroy});



#----------------------------------------------------
# Phase 5: Accelerators, mnemonics, command-line info
#----------------------------------------------------

my $menu_file_m = $menu_file->cget(-menu);
$menu_file_m->entryconfigure(2, -accelerator => 'Ctrl+Q');
$app->bind('<Control-q>' => sub {exit});


if ($main::redmode ne "auto") {LoadParams($mainfile)}

# We do this rather than simply $frame_1_e->focus to eliminate the Perl
# "single use" warning message.

$frames[1]->focus;

#wait for user only in interactive mode

 #end of if not AUTO
}

#wait for user only in interactive mode




 if ($task =~ /ccdproc$/) {
   my $plist=$inlist;
  #strip the @
  $plist=~s/\@//;
  my $nopath=$plist;
  #take only filename without path
  my @string=reverse(split(/\//,$nopath));
  $nopath=$string[0];
  # add out-
  $nopath="out-".$nopath;
  #substitute xxx with out-xxx in the complete filename
  $outlist=$plist;
  $outlist=~ s/$string[0]/$nopath/;
  $outlist="@".$outlist;
} 


if ($main::interface eq 'gui' && $main::redmode eq "interactive") {$app->waitWindow} else {$ret=&goAction}

$$routlist=$outlist;

return $ret;

} #end of mkForm (the main procedure)


sub goAction {

  my $ret=0;

  if (-e "doit.cl") {system ("rm doit.cl") }

SaveParams($mainfile, "go"); #save current script file and check for list


if ($task =~ /ccdproc$/ ) {
           my $l="... Processing frames from $inlist";
	   if (! -e $inlist) { $l=$l."  (NOT FOUND !!!)"};
           $l=$l." to $outlist\n";
           $l=$l."       Copy of FITS files and header modification\n";
	   $l=$l."       IRAF task called: $proc\n";

              &main::mkLog($l);
	
	      
         if ($main::redmode ne "listonly") {
               my $wclock0=time;
	       my $cpu0=(times)[0];              
	       my $ncop=&CopyFITS("doit.cl");
               my $cpu1=(times)[0];
               my $wclock1 = time; 	   
              printf "$ncop FITS files copied, it took: %.2fs (user), %.2fs (CPU)\n", $wclock1-$wclock0,$cpu1-$cpu0;
           }
  }

if ($task =~ /combine$/) {
     SaveParams($mainfile, "go"); #save current script file and check for list
      
       my $o=$outlist;
       
       # $outlist is not necessarily a list, may be the root name (e.g. Flat)
       
       if ($o !~ /@/ && $o !~ /\.fits/) {$o=$o."*.fits"};

           my $l="... Combining frames from $inlist ";
           if (! -e $inlist) { $l=$l."  (NOT FOUND !!!)"};
           $l=$l." to $o\n";
           $l=$l."    WITH scale: $scalmod, combine: $combmod \n";
	   $l=$l."       IRAF task called: $proc";
	
	      &main::mkLog($l);
	
	  
         if ($main::redmode ne "listonly") {
         #before combining, we delete the existing output files (alread MSCRED will append on them the new files !!!) 
             my $dcmd="del (\"$o\", verify=no)";
              open (FILE, ">>doit.cl");
                   print FILE "$dcmd\n";
              close (FILE);
          }
     }


     if ($inlist eq "") {$ret=0 ;print "$task was not executed\n";return $ret}


# Append scripts in order to avoid to call Iraf twice
     system ("cat $mainfile >> doit.cl") ;

if ($main::redmode ne "listonly") {
#ok, start IRAF
               my $wclock0=time;
	       my $cpu0=(times)[0];
                   $ret=CallIRAF("doit.cl");
               my $cpu1=(times)[0];
               my $wclock1 = time;
               printf "IRAF completed $task, it took: %.2fs (user), %.2fs (CPU)\n", $wclock1-$wclock0,$cpu1-$cpu0;
     }

# if $dofringe was set, then match and subtract fringe frame
if ($task =~ /ccdproc$/ && $task =~ /obj/ && $dofringe == 1) {
   require PDL::PDLfuncs;
   PDLfuncs::irafFringe($outlist, $fringes,$def_vals{FRWIND}->{Value}) 
}


return $ret;

} #end goAction

sub clearAction {
    foreach my $i (0..$#nfields) {
	$frames[$i+1]->delete('0', 'end');
    }
} #end clearAction

sub fileAction {
    my $oper=shift;
    &main::getFile($oper,\$fname,".cl");
    #print "loading parameters from file $fname \n";
    if ($oper eq 'load') {LoadParams($fname, ".cl");}
    if ($oper eq 'save') {SaveParams($fname, ".cl");}
} #end fileAction


sub LoadParams
  {
     foreach my $i (0..$#nfields) {
       $frames[$i+1]->delete('0','end');
       $frames[$i+1]->insert('0',$def_vals{$nfields[$i]}->{Value});
    }

if ($main::redmode ne "interactive") {


  return;
}

my $file = shift;

&clearAction; #clear the window

if ( not open (FILE, $file)) {
	$main::top->Dialog(
            -title  => 'File Not Found',
            -text   => "Can't load $file",
            -bitmap => 'error',
        )->Show;
	return
     }
while (<FILE>) {
chomp($_);

         my @vals =split("=",$_,2);
	 my @string=split('\.',$vals[0],2);
	 my $field=$string[1];
         $field =~ tr/ //d;       
	 foreach my $i (0..$#nfields) {
	     if ($field eq $def_vals{$nfields[$i]}->{Name}) {
                 my $val=$vals[1];
#--------------------------------------------
#HERE I PUT SOME CASES WHERE VALUES LOADED FROM FILE ARE MODIFIED BY THE PROGRAM
# Get rid of " and spaces 
                 $val =~ tr/"//d;
                 $val =~ tr/ //d;

#if making zerocombine, then zero, dark and flat must be set to NO
               if ($task eq 'zeroccdproc') {
                    if ($field eq 'zerocor') {$val='no'} 
                    elsif ($field eq 'darkcor') {$val='no'} 
                    elsif ($field eq 'flatcor') {$val='no'} 
                   } 

#if making darkcombine, then dark and flat must be set to NO
               elsif ($task eq 'darkccdproc') {
                    if ($field eq 'darkcor') {$val='no'} 
                    elsif ($field eq 'flatcor') {$val='no'} 
                   } 

#if making flatcombine, then flat must be set to NO
               elsif ($task eq 'flatccdproc') {
                    if ($field eq 'flatcor') {$val='no'} }
 
#if $inlist was given in the arguments and is not blank (if blank use value already given in file),
# then use that value (use the fact that in IRAF lists are given as the first argument)
               if ($i==1 && $inlist ne "") {$val=$inlist}
#same for output
               if ($outlist ne "" && $i==2) {$val=$outlist}	
#--------------------------------------------
#ok, display values

	         $frames[$i+1]->delete('0', 'end');
                 $frames[$i+1]->insert('0', $val);
		}
	}
   }
		 
 close (FILE);

#The first entry in the list contains the list of images: store it into the global $inlist variable
$inlist=$frames[1]->get;
$outlist=$frames[2]->get;

#This is now the new current file
$mainfile=$file;

  } #end LoadParams


sub SaveParams
  {
my $file = shift;
my $action=shift; 

my $value="";

$dofringe=0;


 open (FILE, ">$file");

# In IRAF, the input files must be given as an argument of the task
# when calling it, otherwise it will ask for it.
# So,  print it in the form ....

foreach my $i (0..$#nfields) {
#read value from entry; 
       if ($main::redmode eq "interactive") {$value=$frames[$i+1]->get} 
       else {$value=$def_vals{$nfields[$i]}->{Value}}

#get the name of the entry 
        my $field=$def_vals{$nfields[$i]}->{Name};
#The first field is the list of images: copy it into the global $inlist variable
 
       if ($i == 0 && $main::redmode eq "interactive") {$inlist=$value};

       if ($task =~ /combine/) {
           if ($field eq "combine") {$combmod=$value}
           elsif ($field eq "scale") {$scalmod=$value}
        }
#//////////////////////////////////////////////////////////////////////////////
# This section manipulates what is sent to the IRAF script

#(1) Add "new" parameters to the form or check "standard" ones
  if ($task =~ /ccdproc$/ && $field eq "plotfile") {
           print FILE "mscred.$field = \"$value\" \n";
              if ($action eq "go" && -e $value) {system ("rm $value")}
           next; #skips to next value
    }
  elsif   ($task =~ /ccdproc$/ && $field eq "frwindow") {
       #this is only used internally by PDL 
       next; #skips to next value
    }   
  elsif ($task =~ /ccdproc$/ && $task =~ /\Aobj/) {
        if ($field =~ /fringecor/) { 
#check if fringes should be subtracted from the science frames
#but then put no fringe correction in the script (PERL will do it...)
            if (lc($value) =~ /y/) {$value="no";$dofringe=1};
	}
	elsif ($field =~ /fringe$/) {
	     $fringes=$value;
	}      
  }
  elsif ($field eq "output") {
     if ($task !~ /ccdproc$/) {$outlist=$value}  else {
              if ($main::interface eq 'gui' && ($main::redmode eq "interactive" || ($main::redmode eq "listonly" && $main::debug eq 'y')) ) {
                      $frames[$i+1]->delete('0', 'end');
                      $frames[$i+1]->insert('0', $outlist);
               }
           }
      }

# (2) IRAF wants strings, sections and * between quotes, but keep out booleans
	if ( ($value =~ /[a-z]|[A-Z]/ && $value !~ /\b(Y|y|yes|YES|no|NO|n|N)\b/) || ($value =~ /\*/) ||($value =~ /\[|\]/) || ($value eq "")) {
                       print FILE "$proc.$def_vals{$nfields[$i]}->{Name} = \"$value\" \n"
               } 
	        else {print FILE "$proc.$def_vals{$nfields[$i]}->{Name} = $value\n"};
 

#//////////////////////////////////////////////////////////////////////////////
 } #end of loop on names

#//////////////////////////////////////////////////////////////////////////////
# This section writes in the IRAF script some parameters which are not displayed in the form
  #write some parameters not given in the interface
      if ($task =~ /ccdproc$/) {
            print FILE "$proc.ccdtype = \"\"\n"; #do not select ccd types
            print FILE "$proc.interactive = no\n"; #IRAF can't be used in interactive mode from external calls
            print FILE "mscred.backup = \"none\" \n";
      }
      elsif ($task =~ /combine$/) {
            print FILE "$proc.ccdtype = \"\"\n"; #Do not use ccdtype in combination
            if ($task !~ "combine") {print FILE "$proc.process = no\n"}; #This should have already been done by CCDPROC
#            print FILE "$proc.clobber = yes\n"; #Overwrite image if exists (not implemented ?)                                    
        } 
#\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\      

#... and then give the entry number 1 as an argument of the task

if ($task =~ /combine$/) {
        print FILE "$proc(\"$inlist\", output=\"$outlist\")\n"
    } 
#in case of ccdproc we already copied raw (links) to output (true) files, so we need $outlis
elsif ($task =~ /ccdproc$/) {print FILE "$proc(\"$outlist\", output=\"$outlist\")\n";}



close (FILE);

#check that the input list really exists
if ($main::redmode eq "interactive" && $action eq "go" ) {
      # in order to check if the file exists, we must get rid of the @
                my $flist=$inlist;
                $flist =~ s/\@//;
	        if (not -e $flist) {
	       		$main::top->Dialog(
                            -title  => 'ERROR !!!',
                            -text   => "Can't find the input list ($flist),\n please make it",
                            -bitmap => 'error',
                        )->Show;
			&main::mklistAction;
		      }
            }


#make outlist for ccdproc (better to overwrite it always)
#However if mkfringe, then inlist=outlist
if ($task =~ /ccdproc$/  && $task !~ /\Amkfringe/) {&mkOutList;}   


#This is now the new current script file (user may have changed the default name)
$mainfile=$file;



} # end SaveParams


sub mkOutList {
#create an output list


#$inlist=$frames[1]->get;
#$outlist=$frames[2]->get;

my $flist=$inlist;
# in the IRAF environment , @ just means that it is a list (it is not included in the filename)
$flist=~s/\@//;

#read the list of input files
open (IFILE, $flist);
   my @raw=<IFILE>;
close (IFILE);

$flist=$outlist;
$flist=~s/\@//;
open (OFILE, "> $flist");

foreach my $raw (@raw) {
   chomp($raw);
   if ($raw eq "") {next}
   $raw =~ s/\.fits$//;
   $raw=$raw."Oir.fits";
   print OFILE "$raw\n";
 }


close (OFILE);


} #end sub mkOutList

sub CopyFITS {

  use File::Copy;

my $fscript=shift;

my $ncopied=0;


my $flist=$inlist;
# in the IRAF environment , @ just means that it is a list (it is not included in the filename)
$flist=~s/\@//;

#read the list of files
open (IFILE, $flist);
   my @raw=<IFILE>;
close (IFILE);



$flist=$outlist;
# in the IRAF environment , @ just means that it is a list (it is not included in the filename)
$flist=~s/\@//;

#read the list of files
open (OFILE, $flist);
   my @proc=<OFILE>;
close (OFILE);


open (SFILE, "> $fscript");
for (my $i=0; $i<scalar(@raw); $i++ ) {
    my $ds="";
    my $rf=$raw[$i];
    my $pf=$proc[$i];
    chomp($rf);
    chomp($pf);
    
    # If file exists then do not overwrite IF object processing (files may have been already 
    # created for superflat)
    if (-e $pf && $task =~ /\Aobject/ && $main::aSFLAT eq 1) {next}
    
    #make a copy of the raw file 
    
    if ($main::redmode ne "listonly") {    
       copy ($rf, $pf); 
       #check the section, if flipped then 
       #add a line to the script file (could also change directly the header...)
       my $flipped=&getdatasec($pf, \$ds, 1);
       if ($flipped) {print SFILE "hedit (\"$pf\", \"DATASEC\", \"$ds\", show=no, update=yes, verify=no ) \n"}
    }
  
    $ncopied++;

}

close (SFILE);

return $ncopied;

} #end sub copyFITS



sub CallIRAF
{
  my $script=shift;
  if (not -e "./login.cl")  {

      $my $bin=$FindBin::Bin;
      system ("$main::BIN/Iraf/mklogin.csh");
          my $l="Warning: login.cl not found, a new one was created";
                 &main::mkLog($l,"W","internal")
     }

  open (SFILE, ">> $script");

  print SFILE "logout\n";

  close (SFILE);

  #my $cmd="cl < $script";
  if (-e "tmp.log") {system ("rm tmp.log")}
  my  $cmd="cl < $script > tmp.log"; # This may be used to create a logfile
 
  #system ($cmd);

my $nerr=0;

if ($main::redmode ne "listonly") {
 my $ustop=0;


   # add error processing
           my $pid = open(KID_TO_WRITE, "|-");
           $SIG{ALRM} = sub {die "whoops,  pipe broke" };

           if ($pid) {  # parent
               my @data;
               for (@data) {
                   print KID_TO_WRITE;
               }
               if (!close(KID_TO_WRITE)) {warn "IRAF stopped !!\n"; $ustop=1} 

           } else {     # child
             #  ($EUID, $EGID) = ($UID, $GID);
               exec($cmd)
                   || die "can't exec $cmd: $!";
               # NOTREACHED
           }



  open (OFILE, "tmp.log");
     my @outsoft=<OFILE>;
  close(OFILE);

my @errs=();
my @warns=();


if ($ustop == 1) {
  @errs=("ERROR: process stopped by user !");
}
else {  
  @errs=grep(/ERROR/, @outsoft);
  @warns=grep(/Warning/, @outsoft);
} 

 my $nwarn=scalar(@warns);
 $nerr=scalar(@errs);

  if ($nerr !=0 || $nwarn !=0) {
     if ($nerr !=0) {
         foreach my $err (@errs) {
            my @string=split(/ERROR:/,$err,2);
             my $l="ERROR: $string[1]";
	      $l =~ tr/\n//d;
	         &main::mkLog($l);
          }
      }
      
       if ($nwarn !=0) {
         foreach my $warn (@warns) {
            my @string=split(/Warning:/,$warn,2);
               my $l="Warning: $string[1]";
	       $l =~ tr/\n//d;
	          &main::mkLog($l);
          }
      }
          	  
   }
} 

return $nerr;

} #end sub CallIRAF

sub msgHelp
{

     my($topic, $x, $y) = @_;
    if ($topic eq '') {return;};

my $idx=0;
my $msg;

   foreach my $i (0..$#nfields) {if ($topic eq $frames[$i+1]) {$idx=$i; last}}


$msg= &iraf_help::getHelp($task, $idx);

#if ($idx >0 && defined $iraf_help::helpTopics[$idx]) {
#   $msg=$iraf_help::helpTopics[$idx]} else
#   {$msg="Sorry, no help !"}

    $hdlg->Subwidget('message')->configure(
        -text => "Information on $proc:$def_vals{$nfields[$idx]}->{Name}\n\n$msg");
    $hdlg->Show;
} #end sub Help

sub getdatasec
#read DATASEC and check whether it is flipped
{
my $input=shift;
my $newsec=shift;
#if $chflip=1 then DATASEC will be corrected if flipped
my $chflip=shift;

my $flipped=0;

my $fix80=80;
open (CCD, $input);
my $header=<CCD>; 
my $length_header=length($header);
my $offset=0;

while ($offset < $length_header)
   {$_=substr($header,$offset,$fix80);
   if (/DATASEC =/ ) {
   my @string=split(/'/,$_,3);
   my $s=$string[1];
   $s=~tr/[\[\]]//d;
   $_=$s;
   my @n=/(\d{1,4}):(\d{1,4}),(\d{1,4}):(\d{1,4})/;
   if ($chflip eq 1) {
      my $t;
      if ($n[1] < $n[0]) {$t=$n[0]; $n[0]=$n[1]; $n[1]=$t; $flipped=1}
      if ($n[2] > $n[3]) {$t=$n[2]; $n[2]=$n[3]; $n[3]=$t; $flipped=1}
   }

   $$newsec="[$n[0]:$n[1],$n[2]:$n[3]]";
  last;

 }
 $offset+=$fix80;

}
close(CCD);

return $flipped;

} #end getdatasec


sub DoArith 
{
my $op1=shift;
my $op=shift;
my $op2=shift;
my $result=shift;

my $string="";

#$string=$string . "mscarith.operand1 = $op1\n";
#$string=$string . "mscarith.op = $op\n";
#$string=$string . "mscarith.operand2 = $op2\n";
#$string=$string . "mscarith.result = $result\n";
$string=$string . "mscarith.extname = \"\"\n";
$string=$string . "mscarith.title = \"\"\n";
$string=$string . "mscarith.divzero = 0 \n";
$string=$string . "mscarith.hparams =  \"\"\ \n";
$string=$string . "mscarith.pixtype =  \"\"\ \n";
$string=$string . "mscarith.calctype =  \"\"\ \n";
$string=$string . "mscarith.verbose =  no \n";
$string=$string . "mscarith.noact =  no \n";

$string=$string . "mscarith( \"$op1\", \"$op\",  \"$op2\", \"$result\")\n";


return $string;

} #end sub DoArith

sub DoMedian
{
my $op1=shift;
my $op2=shift;
my $xwin=shift;
my $ywin=shift;
my $string="";


$string=$string . "median.zloreject = INDEF\n";
$string=$string . "median.zhireject = INDEF\n";
$string=$string . "median.boundary = \"nearest\"\n";
$string=$string . "median.constant = 0.\n";
$string=$string . "median.verbose = no\n";

$string=$string . "median( \"$op1\", \"$op2\", $xwin, $ywin)\n";

return $string;

} #end sub DoMedian





sub build_name {

#*********************************************************
# THIS ROUTINE DEFINES THE PARAMETERS FOR EACH TASK 
# AS WELL AS THE DEFAULT VALUES, WHICH MAY BE CHANGED HERE
# AS WELL AS IN THE SUB ** MkConfig **
#*********************************************************


my $task=shift;

my $prc='';


#/////////////////////////////////////////////////////////////////////
# Here $task is the task of the procedure, that is what it should do
#      $prc is the procedure which is actually called
#////////////////////////////////////////////////////////////////////

$go="Run ";


if ($task =~ /combine$/) {

%def_vals= (
INPUT => { Name => 'input', Value => ''},
OUTPUT => { Name => 'output', Value =>''},
COMBINE => {Name => 'combine', Value => 'aver'},  #may be medianaverage
REJECT => { Name => 'reject', Value => 'minmax'}, 
DELETE => { Name => 'delete', Value => 'no'},                           
SCALE=>{ Name => 'scale', Value => 'none'},      #may be none|mode|median|mean|exposure       
STATSEC => { Name => 'statsec', Value => ''},     
NLOW => { Name => 'nlow', Value => '0'},
NHIGH => { Name => 'nhigh', Value => '1'},
NKEEP => { Name => 'nkeep', Value => '1'},
NLOW => { Name => 'nlow', Value => '0'},
MCLIP => { Name => 'mclip', Value => 'yes'}, #set to 'no' to clip with average, 'yes' with median
LSIGMA => { Name => 'lsigma', Value => '3'},
HSIGMA => { Name => 'hsigma', Value => '3'},
RDNOISE => { Name => 'rdnoise', Value => '0'},
GAIN => { Name => 'gain', Value => '1'},
SNOISE => { Name => 'snoise', Value => '0'},
PCLIP => { Name => 'pclip', Value => '-0.5'},
BLANK =>{ Name => 'blank', Value => '0'},
);

 if ($task =~/\Asflat/) {$prc="flatcombine"} 
 elsif ($task =~/\Aobject/ || $task =~/\Amkfringe/) {$prc="combine"} 
 else {$prc=$task}

}
elsif ($task =~ /ccdproc$/) {

#the CCDPROC parameters in MSCRED

%def_vals= (
INPUT => { Name => 'input', Value =>$inlist},
OUTPUT => { Name => 'output', Value => $outlist},
NOPROC => { Name => 'noproc', Value => 'no' },  
DOOVERSCAN => { Name => 'overscan', Value => ''},         
DOTRIM => { Name => 'trim', Value => ''},                           
FIXPIX => { Name => 'fixpix', Value =>'no'},          
DOBIAS => { Name => 'zerocor', Value => 'no'},
DODARK => { Name => 'darkcor', Value => 'no'},
DOFLAT => { Name => 'flatcor', Value => 'no'},
DOSFLAT => { Name => 'sflatcor', Value => 'no'},
DOFRINGE => { Name => 'fringecor', Value => 'no'},
OSSEC => { Name => 'biassec', Value => ''},          
TRIMSEC => { Name => 'trimsec', Value => ''},
FIXFILE => { Name => 'fixfile', Value => ''},
BIAS => { Name => 'zero', Value => ''},
DARK => { Name => 'dark', Value => ''},
FLAT => { Name => 'flat', Value => ''},
SFLAT => { Name => 'sflat', Value => ''},
FRINGE => { Name => 'fringe', Value => ''},
FRWIND => { Name => 'frwindow', Value => ''},
MINREP => { Name => 'minreplace', Value => '1'},
OSFUNC => { Name => 'function', Value => 'legendre'}, #function fitting the overscan (legendre|chebyshev|spline1|spline3)
OSORDER => { Name => 'order', Value => '3'},
OSSAMPLE => { Name => 'sample', Value => '*'},
NAVERAGE => { Name => 'naverage', Value => '-1'}, #number of points to average (>0) or median (<0) to fit the overscan
NITERATE => { Name => 'niterate', Value => '1'},
LREJ => { Name => 'low_reject', Value => '3'},
HREJ => { Name => 'high_reject', Value => '3'},
GROW => { Name => 'grow', Value => '0'},
OSPFILE => { Name => 'plotfile', Value => 'no'}  #plot the overscan fit
);

 $prc="_ccdtool";
}

@nfields=keys %def_vals; #store here the names of the fields


return $prc;


}  #end build_name


sub mkConfig {
  my $step=shift;

if (! -e "batch.par") {print "COULD NOT OPEN batch.par !!!\n"; return}

open (IFILE, "batch.par");

   my @all=<IFILE>;
   chomp(@all);
close (IFILE);

 # extract all  info for the given step from the batch configuration file
  my @allsteps=("misc","mask","overscan","trim"); #these are the steps to be performed ALWAYS

#define which steps should be performed

  if ($step =~ /\Azero/) {$step=~s/zero/bias/; push (@allsteps ,"bias")}
  elsif ($step =~ /\Adark/) {push (@allsteps, "bias","dark")}
  elsif ($step =~ /\Aflat/) {push (@allsteps, "bias","dark","flat")}
  elsif ($step =~ /\Asflat/) {push (@allsteps, "bias","dark","flat","sflat")}
  elsif ($step =~ /\Amkfr/) {push (@allsteps,  "bias","dark","flat","sflat","mkfringe")}
  elsif ($step =~ /\Aobject/) {push (@allsteps, "bias","dark","flat","sflat","mkfringe","object")}

  my $dobias="no";
  my $dotrim="no";
  my $dodark="no";
  my $doflat="no";
  my $dosflat="no";
  my $dofringe="no";
  my $combtype;
  my $combscale;
  my $combrej;   
  my $doosplot="no";
  my $usemask='no';

  #my $doobjplot="no";

  my ($ossec, $doos, $osord, $trimsec, $nbias, $ndark, $nflat, $nsflat, $nfringe, $frwind, $nmask);

  
#loop on each step
  foreach my $dostep (@allsteps) {

  $combtype="none"  ;
  $combscale="none" ;
  $combrej="none";

#extract from the configuration array (@all) the entries for the current step
# the \A is needed to discriminate between flat and sflat
   my @arr=grep(/\A$dostep/,@all);

#loop on entries
   foreach my $entry (@arr) {
      chomp($entry);
      my @string=split(/ /,$entry,2);
    #this is the value
      my $val=$string[1];
        $val =~ tr/ //d;
       #separate step and field
        @string=split(/:/,$string[0],2);

 if ($string[1] eq "combine") {
      $combtype=$val;
      if ($combtype =~ /avsig/i) {   #Translate for IRAF the "avsigclip" option
            $combrej="avsigclip";
            $combtype="average"
        };
   };
 if ($string[1] eq "scale") {$combscale=$val};

        if ($dostep eq "overscan") {
            if ($string[1] eq "section") {$ossec=$val}
            elsif ($string[1] eq "apply") {$doos=$val}
            elsif ($string[1] eq "order") {$osord=$val}
	    elsif ($string[1] =~ /plot/i) {$doosplot=$val}	    
         } elsif ($dostep eq "trim") {
	   if ($string[1] eq "section") {$trimsec=$val} 
	   elsif ($string[1] eq "apply") {$dotrim=$val}
	 }   elsif ($string[0] eq 'mask') {
                if ($string[1] eq 'use') {
                   $usemask=$val;
                 }
               elsif ($string[1] eq 'mask') {
                  $nmask=$val;
               }
               next;
           }  else {
          #for each step, name of output list should be stored in case it is needed later
          if ($dostep eq "bias") {
               if ($string[1] eq "apply") {$dobias=$val} 
               elsif ($string[1] eq "output") {$nbias=$val."*"}
	        if ($step =~ /\Abias/) {
		          $dobias="no";
	        } 
             }
             elsif ($dostep eq "dark") {
              if ($string[1] eq "apply") {$dodark=$val} 
               elsif ($string[1] eq "output") {$ndark=$val."*"}
	       if ($step =~ /\Adark/) {
	                    $dodark="no";
			    }
             }
             elsif ($dostep eq "flat") {
               if ($string[1] eq "apply") {$doflat=$val} 
               elsif ($string[1] eq "output") {$nflat=$val."*"}
	        if ($step =~ /\Aflat/) {
	            $doflat="no";
		    $dosflat="no";
		 }
             }
             elsif ($dostep eq "sflat") {
                     if ($string[1] eq "apply") {$dosflat=$val} 
                     elsif ($string[1] eq "output") {$nsflat=$val."*"}
		     if ($step =~ /\Asflat/) {
		        $dosflat="no";
		      }
             }
	     elsif ($dostep eq "mkfringe") {
                     if ($string[1] eq "apply") {$dofringe=$val} 
                     elsif ($string[1] eq "output") {$nfringe=$val."*"}
		     elsif ($string[1] eq "smooth") {$frwind=$val}
		     if ($step =~ /\Amkfr/) {
		         $dofringe="no";
		     }
             }
	      elsif ($dostep eq "object") {
		}
         #Store input and output lists only for the last step (e.g. if $step=flatcombine && $dostep=flat)
           if  ($step =~ /\A$dostep/) {
               if ($inlist !~ /\w+/ && $string[1] eq "input") {$inlist=$val}
               elsif ($string[1] eq "output") {
	             $outlist=$val;
		  }
           }
         }
     }  #end loop
  } #end loop on steps


#_ccdtool
  if ($step =~ /ccdproc$/) {

        $def_vals{INPUT}->{Value}=$inlist;
        $def_vals{OUTPUT}->{Value}='';
        $def_vals{DOOVERSCAN}->{Value}=$doos;
        $def_vals{DOTRIM}->{Value}=$dotrim;
        $def_vals{DOBIAS}->{Value}=$dobias;
        $def_vals{DODARK}->{Value}=$dodark;
        $def_vals{DOFLAT}->{Value}=$doflat;
        $def_vals{DOSFLAT}->{Value}=$dosflat;
        $def_vals{DOFRINGE}->{Value}=$dofringe;
        $def_vals{FIXPIX}->{Value}=$usemask;
        $def_vals{OSORDER}->{Value}=$osord;
        $def_vals{OSSEC}->{Value}=$ossec;
        $def_vals{TRIMSEC}->{Value}=$trimsec;
        $def_vals{BIAS}->{Value}=$nbias;
        $def_vals{DARK}->{Value}=$ndark;
        $def_vals{FLAT}->{Value}=$nflat;
        $def_vals{SFLAT}->{Value}=$nsflat;
        $def_vals{FRINGE}->{Value}=$nfringe;
        $def_vals{FRWIND}->{Value}=$frwind;
        $def_vals{FIXFILE}->{Value}=$nmask;
        $def_vals{OSPFILE}->{Value}=$doosplot;

#Some consistency checks.....
if ($osord <=1) {$def_vals{OSORDER}->{Value}=1};
if ($ossec eq '') {$def_vals{DOOVERSCAN}->{Value}='no'};
if ($trimsec eq '') {$def_vals{DOTRIM}->{Value}='no'};



}

#combine
   elsif ($step =~ /combine$/) {

       $def_vals{INPUT}->{Value} = $inlist;
       $def_vals{OUTPUT}->{Value} = $outlist;
       $def_vals{COMBINE}->{Value} = $combtype;
       $def_vals{REJECT}->{Value} = $combrej, 
       $def_vals{SCALE}->{Value} = $combscale;   
   
   }



}  #end sub MkConfig




1;





