# Last modified: May 5 2001

use Getopt::Long;


use PDL;
use PDL::Math;

use File::Basename;

use Tk;
use Tk::BrowseEntry;
use Tk::Table;
use Tk::Dialog;

use String::Approx 'amatch', 'adistr';

use Common::misc;

use strict;

use vars qw/@header @frames $dbfile $odbfile @log @newtab $top/;

use vars qw /@nels @xmin @xmax @ymin @ymax $lxmin $lxmax $lymin $lymax $nr $c1 $c2 $thresh/;
my $DEBUG=0;

my %optctl = ();
&GetOptions (\%optctl, "input|i=s", "output|o=s", "radius|r=s");

# The name of the database file
my $input='setdata.db';
# The name of the reorganized database file
my $output='setdata_n.db';
#radius is the radius (half width) of each bin in arcmin !
my $radius=15;

$input=$optctl{'input'} if (defined $optctl{'input'});
$output=$optctl{'output'} if (defined $optctl{'output'});
$radius=$optctl{'radius'} if (defined $optctl{'radius'});

warn "--> $input -> $output\n" if ($DEBUG);

&Init();
checkdb($input, $output, $radius);

sub checkdb {

 $top=MainWindow->new(-name => 'Check DB', -title=>"Check DB");
  
  my $radbin;
  ($dbfile, $odbfile, $radbin)=@_;
  
  
  
  $thresh= float [$radbin/60./15., $radbin/60.]; #convert from arcmin to hours/degrees
  #High values => few groups 
  #Low values =>  many groups
  
  if (!open (DBFILE, $dbfile)) {	
    $main::top->Dialog(
		       -title  => 'ERROR !!!',
		       -text   => "Can't open input file ($dbfile)",
		       -bitmap => 'error',
    )->Show;
    return;
  };
  
  @log=();
  @header=();
  my $skip=0;
  my $rhead=0;
  
  
 ROW: while (<DBFILE>) {
    chomp($_);
    my $row=$_;   
    if ($row=~ /^</) { 
      if ($row=~/^<\w+>/) {
	if ($row =~/HEADER/i) {$rhead=1}
	$skip=1       
      }
      if ($row=~/<\/\w+>/) {$skip=0}
      next ROW;
    } 
    
    if ($rhead==1) {@header=split(/$DBDEL/,$row);$rhead=0; push (@log, $row)} 
    
    next ROW if ($skip==1);
    
    push (@log, $row);
  }
  close (DBFILE);
  
  for my $i (0..$#header) {
    my $h=$header[$i];
    $h =~ tr/ //d;
    $header[$i]=lc($h);
  }
  
  
  #Note: it is more convenient to keep the entries of  @flog as strings, not as arrays,
  # since so we can use grep to filter the array
  
  #Remove from the array the header and the entries for dark, bias or flat
  
  my @flog=grep(!/bias|dark|flat|dome|sky|^\s$/i,@log);
  shift @flog;			#remove the header line
  
   
  # ======= Foreach subsample compute the histogram of the coordinates and find peaks
  
  my $nidx=-1;
  
  #The following loop will:
  # - Loop on the @onames array
  # - Scan the @flog array for all the rows containing the current name
  # - Build the PDL $coords containing the coords for these rows
  # - Build clusters
  # Output: @newtab: array of new names , coordinates  and indexes
  
  #alternative approach:
  # 1. find the histogram of ALL coordinates => peaks with the given threshold
  # 2. assign each name to a given peak
  # 3. change the names (a probability could be even given....)
  
  my $nrows=$#flog+1;
  $c1= zeroes(float,$nrows);
  $c2 = zeroes(float,$nrows);
  $nr = zeroes(short,$nrows);

  my $r=0;
  
 my @names=();

  foreach my $row (@flog) {
    my @fields=split(/$main::DBDEL/,$row);
    my $c=0;
    
    foreach my $field (@fields) {
      if ($header[$c] eq 'nr') {set $nr,$r, $field}
      elsif ($header[$c] eq 'object') {push @names, $field}
      elsif ($header[$c] eq 'ra') {my @str=split(/:/,$field,3);
				   my $val=$str[0]+$str[1]/60.+$str[2]/3600.;
				   set $c1,$r, $val;
				 }
      elsif ($header[$c] eq 'dec') {my @str=split(/:/,$field,3);
				    my $sig=1;
				    if ($str[0]=~/-/) {$sig=-1};
				    my $val=$sig*(abs($str[0])+$str[1]/60.+$str[2]/3600.);
				    
  				    set $c2,$r, $val;
				    last}
      $c++;
    }				#end loop on fields
    
    $r++;
  }
  
  #now we have all the coordinates 
  my $atr=1000/60/15; 
  my $dtr= 1000/60;
  
  my $ast=$atr;
  my $dst=$dtr;

    my ($amin, $amax, $dmin, $dmax);
    
    $amin=min($c1); $amax=max($c1);
    $dmin=min($c2); $dmax=max($c2);


    $amin-=$ast;
    $amax+=$ast;
    my $abins= int(($amax-$amin)/$ast+.5) ;
    

    $dmin-=$dst;
    $dmax+=$dst;
    my $dbins=int(($dmax-$dmin)/$dst+.5);
  

    #Compute the histogram of the coordinates and find clusters
    my $h = histogram2d($c1, $c2, $ast, $amin ,$abins , $dst, $dmin, $dbins);
    #print "h->$h\n";		#we could make a 3d plot %-)

my ($xnz, $ynz) = whichND($h !=0);
my $dnz=$ynz->getdim(0)-1;

my $xc1=$xnz*$ast+$amin;
my $xc2=$ynz*$dst+$dmin;

#my $xh=$h->index2d($xnz, $ynz);


@nels=(); @xmin=(); @xmax=(); @ymin=(); @ymax=(); 

hscan ($h, $amin, $dmin, $ast, $dst);

#ok, we have the coordinates reorganized in bins,
 foreach my $j (0..$#xmin) {
  #print "$xmin[$j] / $xmax[$j] ; $ymin[$j] / $ymax[$j] : $nels[$j]\n";
  my $idx=which ($c1 >= $xmin[$j] & $c1< $xmax[$j] & 
       $c2 >= $ymin[$j] & $c2< $ymax[$j]);

#find a common name and store it into $newtab[$j][1]
my ($nname, $fails)=FName(\@names, $idx);

push @newtab, [$idx->nelem, $nname,
    ($xmin[$j]+$xmax[$j])/2., ($ymin[$j]+$ymax[$j])/2., $fails, 
    $nr->index($idx)];
}
   
    my $atab = $top;
    $atab->title('Clusters of names');
    
    # Use a scrollable table (useful for task with many parameters)
    
    my $at  = $atab->Table(-scrollbars => 'e',-fixedrows => 1, -rows => 20);
    $at->pack(-expand=> 1, -fill => 'both');
    
    my $c=0;
    
    $at->put(0,$c++,$at->Label(-text=>"Name",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"RA",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"DEC",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"Found",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"Doubts",-anchor=>'w'),-relief => 'raised');


    $at->configure(-columns => $c) if ($c > $at->cget('-columns'));
    
    my $buttons = $atab->Frame();
    $buttons->pack(-side => 'bottom', -pady => '2', -anchor => 'center');
    #my $buttons_clear = $buttons->Button(-text => 'Clear');
    #my $buttons_load = $buttons->Button(-text => 'Load');
    my $buttons_apply = $buttons->Button(-text => 'Apply');
    $buttons_apply->configure(-command => \&apply_changes);
    #my $buttons_savedef = $buttons->Button(-text => 'Save def.');
    my $buttons_exit = $buttons->Button(-text => 'Close');
    $buttons_exit->configure(-command =>  (sub {$atab->destroy}));
    #$buttons_skip = $buttons->Button(-text => 'Next');
    

    #    $buttons_clear->pack(-side => 'left', -padx => '2');
    #    $buttons_load->pack(-side => 'left', -padx => '2');
    $buttons_apply->pack(-side => 'left', -padx => '2');
    #    $buttons_savedef->pack(-side => 'left', -padx => '2');
    $buttons_exit->pack(-side => 'left', -padx => '2');
    #$buttons_skip->pack(-side => 'left', -padx => '2');
 
    
 foreach my $j (0..$#xmin) {
  my $xpos=0;
 
  my $idx=which ($c1 >= $xmin[$j] & $c1< $xmax[$j] & 
       $c2 >= $ymin[$j] & $c2< $ymax[$j]);

    my $e;
    ${$e}=$at->BrowseEntry(-width =>'30', -variable => \$newtab[$j][1]);
    $frames[$j+1]=${$e};$at->put($j+1,$xpos++,${$e});
  foreach my $k (0..$idx->nelem-1) {
      $frames[$j+1]->insert("end", $names[$idx->at($k)]);
    }

  my $sfield=sexstr($newtab[$j][2]);
   $at->put($j+1,$xpos++,$at->Label(-text => "$sfield", -anchor => 'e'));
  
    $sfield=sexstr($newtab[$j][3]);
   $at->put($j+1,$xpos++,$at->Label(-text => "$sfield", -anchor => 'e'));
   $at->put($j+1,$xpos++,$at->Label(-text =>$newtab[$j][0]
 , -anchor => 'e'));

   $at->put($j+1,$xpos++,$at->Label(-text =>$newtab[$j][-2] , -anchor => 'e'));


  }
    
$frames[1]->focus;

$atab->waitWindow;

}

sub apply_changes
  {
        
    #replace names and coords in the dblog array (should be the last step....)


    foreach my $i (0..$#newtab)  {
      
      my $p=$newtab[$i][$#{$newtab[$i]}] ;
     
      #update the database
      foreach my $j (0..nelem($p)-1) {
	my $idx=$p->at($j);
	if ($DEBUG) {print ">>>>>>> Reading $idx ($newtab[$i][1])/ ";}
	my $row= $log[$idx];

	my @fields=split(/$main::DBDEL/,$row);
        my $c=0;
        my @nrow=();
	
	my @warnings=();
	
        foreach my $field (@fields) {
	  if ($header[$c] eq "object") {$field =~ tr/ //d;
					push @nrow, $newtab[$i][1];
					my $oname=lc($field); my $nname=lc($newtab[$i][1]);
					if ($oname ne $nname) {
				      my $dist=adistr($nname,$oname);
					  push @warnings, "# Name changed (was $field: $dist)";
					}
				      }
	  elsif ($header[$c] eq "ra") {push @nrow, $field} #we display the TRUE coordinates 
	  elsif ($header[$c] eq "dec") {push @nrow, $field}
	  elsif ($header[$c] eq "warnings") {push @warnings, $field}	
	  else {push @nrow, $field}
	  $c++;
	}			#end loop on fields
	my $warning =join " ", @warnings;
	my $nrow=join "$main::DBDEL", @nrow,$warning;
	if ($DEBUG) {print "\n###### idx -  $log[$idx] -> $nrow ##### \n"};
	
	$log[$idx]=$nrow ; 
      }
      
      if ($DEBUG) {print "\n";}
      
    }
    
    #ok, dump the new @log to the file
    
    open (NEWDB, ">$odbfile");
    
    my $now=&main::mkdate;
    print NEWDB "<DATE> $now </DATE>\n";
    print NEWDB "<INFO> Reorganized with radius (arcmin) = $radius </INFO>\n";    

    print NEWDB "<HEADER>\n";
    my $leg=join $main::DBDEL, $log[0];
    print NEWDB "$leg\n";
    print NEWDB "</HEADER>\n";
    
    foreach my $irow (1..$#log) {my $row=$log[$irow]; chomp($row); print NEWDB "$row\n"; }
    
    close (NEWDB);
    
    $main::top->Dialog(
		       -title  => 'CheckDB',
		       -text   => "The file $odbfile has been made",
		       -bitmap => 'warning',
		      )->Show;
    
  }

sub hscan
  {
# ===== Recursively distribute in clusters a group of coordinates

   my $h=shift;
   my ($amin, $dmin, $ast, $dst, $nelems)=@_;

     if ($ast <  $thresh->at(0)/5 && $dst <  $thresh->at(1)/5)
       {
 #  warn "LOOP completed: $nelems / $ast, $dst  ($thresh)";
push @xmin, $lxmin; push @xmax, $lxmax;
push @ymin, $lymin; push @ymax, $lymax;
push @nels, $nelems;
   return 
       }

   my $nast=$ast/5;
   my $ndst=$dst/5;
     
   my ($xnz, $ynz) = whichND($h !=0);
   my $dnz=$ynz->getdim(0)-1;
   my $hx=$xnz*$ast+$amin;
   my $hy=$ynz*$dst+$dmin;

#warn "$xnz\n $ynz\n $hx \n $hy";  
   foreach my $j (0..$dnz) {   

     my $xamin=$hx->at($j); 
     my $xamax=$hx->at($j)+$ast; 

     my $xdmin=$hy->at($j); 
     my $xdmax=$hy->at($j)+$dst; 

     my $idx=which($c1>=$xamin & $c1<$xamax & $c2>=$xdmin & $c2<$xdmax); 
#warn "=========== $xamin $xamax $xdmin $xdmax -> ", $idx->nelem;
     next if ($idx->nelem < 1);
     my $xc1=$c1->index($idx);
     my $xc2=$c2->index($idx);  

     my $abins= int(($xamax-$xamin)/$nast+.5) ;
     my $dbins= int(($xdmax-$xdmin)/$ndst+.5);    
  
#warn "mk histogram for:  $nast, $xamin ,$abins , $ndst, $xdmin, $dbins";
     my $h1 = histogram2d($xc1, $xc2, $nast, $xamin ,$abins , $ndst, $xdmin, $dbins);
     
#  warn "new level with $nast $ndst\n";
$lxmin=$xamin; $lxmax=$xamax;
$lymin=$xdmin; $lymax=$xdmax;

      hscan($h1, $xamin, $xdmin, $nast, $ndst, $idx->nelem);
   }
#warn "LOOP over";
  }



sub sexstr {
  my $value=shift;
  my $sig=$value/abs($value);
  my $f1=int($value);
  my $f3=abs($value-$f1)*60.;
  my $f2=int($f3);
   $f3=($f3-$f2)*60;
   my $if3=int($f3);
    my $value=sprintf ("%02d:%02d:%02d.%.0f",$f1,$f2,$if3,100*($f3-$if3));
    if ($sig==-1 && $f1 ==0)  {$value='-'.$value}
   elsif ($f1 !~ /^-/ && $f1 >=0) {$value='+'.$value}
   
   return($value);
}


sub FName {
# Use approximate pattern matching to 
# find the string that better matches all the names

my ($names, $idx)=@_;

my $best=$$names[$idx->at(0)];
my $fails=0;

 if ($idx->nelem >1) {

# Extract the names for the current range of coordinates
  my @snames=();
   foreach my $k (0..$idx->nelem-1) {
       push @snames, $$names[$idx->at($k)];
   }

 
#== In the first iteration disallow substitutions/deletions
#==   since we want to find the minimum substring contained in all entries 
#==   If it fails, then make a second iteration with subst/deletions ON

  my $rules=["i","S0", "D0"];


 ITER: foreach my $it (1..2) {

# == if it is the 2nd iteration, then enable subst/del
# ==  now we look 
   $rules=["i"] if ($it==2);
    my $score=-1;

  IDX: foreach my $k (0..$idx->nelem-1) {
       my @xsnames=@snames;
       splice(@xsnames, $k,1); #cut off the current (=k) entry 
  
       my $cname=$$names[$idx->at($k)];
   STR:  for (my $l=length($cname);$l>1;$l--) {
       my $pname=substr($cname, 0,$l);

# Find how many entries are matched 
       my @a=amatch($pname, $rules, @xsnames);
       if ($#a > $score) {$score=$#a; #store the number of successfull matches
                $best=$pname;
		#$dist=float(adistr($pname, @xsnames));
	       
     #=== stop if ALL entries are matched
		 last ITER if ($score==$#xsnames) 
             }
      }
     }
  }

# == Check if there are "dubious" cases
  foreach my $entry (@snames) {
    my $dist=adistr(lc($best), lc($entry));
    $fails++ if ($dist !=0) ;
  }

 
};

return ($best, $fails);

}

1;
