use Getopt::Long;


use PDL;
use PDL::Math;

use File::Basename;

use Tk;
use Tk::BrowseEntry;
use Tk::Table;
use Tk::Dialog;


use Common::misc;

use strict;

use vars qw/@header @frames $dbfile $odbfile @log @newtab $top/;

use vars qw /@nels @xmin @xmax @ymin @ymax $lxmin $lxmax $lymin $lymax $c1 $c2  $thresh/;
my $DEBUG=1;

#tolerance is the tolerance in arcmin !


my %optctl = ();
&GetOptions (\%optctl, "input|i=s", "output|i=s", "tolerance|t=s");

my $input='setdata.db';
my $output='setdata_n.db';
my $tol=15;

$input=$optctl{'input'} if (defined $optctl{'input'});
$output=$optctl{'output'} if (defined $optctl{'output'});
$tol=$optctl{'tolerance'} if (defined $optctl{'tolerance'});


&Init();
checkdb($input, $output, $tol);

sub checkdb {

 $top=MainWindow->new(-name => 'Check DB', -title=>"Check DB");
  
  my $tol;
  ($dbfile, $odbfile, $tol)=@_;
  
  
  
  $thresh= float [$tol/60./15., $tol/60.]; #convert from arcmin to hours/degrees
  #High values => few groups 
  #Low values =>  many groups
  
  if (!open (DBFILE, $dbfile)) {	
    $main::top->Dialog(
		       -title  => 'ERROR !!!',
		       -text   => "Can't open input file ($dbfile)",
		       -bitmap => 'error',
    )->Show;
    return;
  };
  
  @log=();
  @header=();
  my $skip=0;
  my $rhead=0;
  
  
 ROW: while (<DBFILE>) {
    chomp($_);
    my $row=$_;   
    if ($row=~ /^</) { 
      if ($row=~/^<\w+>/) {
	if ($row =~/HEADER/i) {$rhead=1}
	$skip=1       
      }
      if ($row=~/<\/\w+>/) {$skip=0}
      next ROW;
    } 
    
    if ($rhead==1) {@header=split(/$DBDEL/,$row);$rhead=0; push (@log, $row)} 
    
    next ROW if ($skip==1);
    
    push (@log, $row);
  }
  close (DBFILE);
  
  for my $i (0..$#header) {
    my $h=$header[$i];
    $h =~ tr/ //d;
    $header[$i]=lc($h);
  }
  
  
  #Note: it is more convenient to keep the entries of  @flog as strings, not as arrays,
  # since so we can use grep to filter the array
  
  #Remove from the array the header and the entries for dark, bias or flat
  
  my @flog=grep(!/bias|dark|flat|dome|sky|^\s$/i,@log);
  shift @flog;			#remove the header line
  
  
  #STEP 1: make subsample by names
  
  my @onames=();
  my @xflog=@flog;
  
#  my $nf=0;
  
#  # This loop will:
#  # - read and split the first row of @xflog
#  # - store the content 'object' field into the @onames array
#  # - remove all rows in @xflog with the same name in 'object'
#  # - stop when @xflog is empty
#  # result: the @onames will contain all names in the db.log as unique entries
  
#  shift @xflog;			#remove the header line
  
#  while(@xflog)  {
#    my @fields=split(/$main::DBDEL/,$xflog[0]);
#    my $xname;
#    my $c=0;
#    foreach my $field (@fields) {
#      chomp ($field);
#      if ($header[$c] eq "object" ) {$field=~ tr/ //d; $xname=$field;push(@onames,lc($xname))}
#      $c++;
#    }
#    $nf=grep(/\b$xname\b/i,@xflog);
#    if ($DEBUG){ print "-> #$xname# $nf $#xflog\n"; }
#    @xflog=grep(!/\b$xname\b/i,@xflog);
    
#  }
  
  
  #STEP 2: foreach subsample compute the histogram of the coordinates and find peaks
  
  my $nidx=-1;
  
  #The following loop will:
  # - Loop on the @onames array
  # - Scan the @flog array for all the rows containing the current name
  # - Build the PDL $coords containing the coords for these rows
  # - Build clusters
  # Output: @newtab: array of new names , coordinates  and indexes
  
  #alternative approach:
  # 1. find the histogram of ALL coordinates => peaks with the given threshold
  # 2. assign each name to a given peak
  # 3. change the names (a probability could be even given....)
  
  my $nrows=$#flog+1;
  
  my $coords=zeroes(float,$nrows,3);
  
  my $r=0;
  
 my @names=();

  foreach my $row (@flog) {
    my @fields=split(/$main::DBDEL/,$row);
    my $c=0;
    
    foreach my $field (@fields) {
      if ($header[$c] eq 'nr') {set $coords,$r, 0, $field}
      elsif ($header[$c] eq 'object') {push @names, $field}
      elsif ($header[$c] eq 'ra') {my @str=split(/:/,$field,3);
				   my $val=$str[0]+$str[1]/60.+$str[2]/3600.;
				   set $coords,$r, 1, $val;
				 }
      elsif ($header[$c] eq 'dec') {my @str=split(/:/,$field,3);
				    my $sig=1;
				    if ($str[0]=~/-/) {$sig=-1};
				    my $val=$sig*(abs($str[0])+$str[1]/60.+$str[2]/3600.);
				    
  				    set $coords,$r, 2, $val;
				    last}
      $c++;
    }				#end loop on fields
    
    $r++;
  }
  
  #now we have all the coordinates 
  
  $c1= $coords->slice(':,(1)') ;
  $c2= $coords->slice(':,(2)') ;
  my $atr=1000/60/15; 
  my $dtr= 1000/60;
 
print ">>>> $c1 \n $c2\n";
  
  my $ast=$atr;
  my $dst=$dtr;

    my ($amin, $amax, $dmin, $dmax);
    
    $amin=min($c1); $amax=max($c1);
    $dmin=min($c2); $dmax=max($c2);


    $amin-=$ast;
    $amax+=$ast;
    my $abins= int(($amax-$amin)/$ast+.5) ;
    

    $dmin-=$dst;
    $dmax+=$dst;
    my $dbins=int(($dmax-$dmin)/$dst+.5);
  

    #Compute the histogram of the coordinates and find clusters
    my $h = histogram2d($c1, $c2, $ast, $amin ,$abins , $dst, $dmin, $dbins);
    print "h->$h\n";		#we could make a 3d plot %-)

my ($xnz, $ynz) = whichND($h !=0);
my $dnz=$ynz->getdim(0)-1;

my $xc1=$xnz*$ast+$amin;
my $xc2=$ynz*$dst+$dmin;

#my $xh=$h->index2d($xnz, $ynz);
print "$xc1 $xc2\n";

warn "start $ast $dst - $thresh";

@nels=(); @xmin=(); @xmax=(); @ymin=(); @ymax=(); 

hscan ($h, $amin, $dmin, $ast, $dst);

#ok, we have the coordinates reorganized in bins,
# now the problem is to assign the names...........

my $tot=0;
  foreach my $j (0..$#xmin) {
  print "$xmin[$j] / $xmax[$j] ; $ymin[$j] / $ymax[$j] : $nels[$j]\n";
  $tot+= $nels[$j];
  my $idx=which ($c1 >= $xmin[$j] & $c1< $xmax[$j] & 
       $c2 >= $ymin[$j] & $c2< $ymax[$j]);
  foreach my $k (0..$idx->nelem-1) {
     print " --- $k => $names[$idx->at($k)]\n";  
  }
  }

print "======> $tot\n";

   
    my $atab = $top;
    $atab->title('Clusters of names');
    
    # Use a scrollable table (useful for task with many parameters)
    
    my $at  = $atab->Table(-scrollbars => 'e',-fixedrows => 1, -rows => 20);
    $at->pack(-expand=> 1, -fill => 'both');
    
    my $c=0;
    
    $at->put(0,$c++,$at->Label(-text=>"Name",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"RA",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"DEC",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"Flag",-anchor=>'w'),-relief => 'raised');
    
    $at->configure(-columns => $c) if ($c > $at->cget('-columns'));
    
    my $buttons = $atab->Frame();
    $buttons->pack(-side => 'bottom', -pady => '2', -anchor => 'center');
    #my $buttons_clear = $buttons->Button(-text => 'Clear');
    #my $buttons_load = $buttons->Button(-text => 'Load');
    my $buttons_apply = $buttons->Button(-text => 'Apply');
    $buttons_apply->configure(-command => \&apply_changes);
    #my $buttons_savedef = $buttons->Button(-text => 'Save def.');
    my $buttons_exit = $buttons->Button(-text => 'Close');
    $buttons_exit->configure(-command =>  (sub {$atab->destroy}));
    #$buttons_skip = $buttons->Button(-text => 'Next');
    
    #If not interactive mode, windows are only used to show the parameters (read only)
    #if ($main::redmode eq "interactive") {
    #    $buttons_clear->pack(-side => 'left', -padx => '2');
    #    $buttons_load->pack(-side => 'left', -padx => '2');
    $buttons_apply->pack(-side => 'left', -padx => '2');
    #    $buttons_savedef->pack(-side => 'left', -padx => '2');
    $buttons_exit->pack(-side => 'left', -padx => '2');
    #$buttons_skip->pack(-side => 'left', -padx => '2');
    #  }
    
 foreach my $j (0..$#xmin) {
  my $xpos=0;
  print "$xmin[$j] / $xmax[$j] ; $ymin[$j] / $ymax[$j] : $nels[$j]\n";
  my $idx=which ($c1 >= $xmin[$j] & $c1< $xmax[$j] & 
       $c2 >= $ymin[$j] & $c2< $ymax[$j]);

#find a common name and store it into $newtab[$j][1]
$newtab[$j][1]=FName(\@names, $idx);

    my $e;
    ${$e}=$at->BrowseEntry(-width =>'30', -variable => \$newtab[$j][1]);
    $frames[$j+1]=${$e};$at->put($j+1,$xpos++,${$e});
  foreach my $k (0..$idx->nelem-1) {
      $frames[$j+1]->insert("end", $names[$idx->at($k)]);
    }

  my $ra=($xmin[$j]+$xmax[$j])/2.;
  my $sfield=sexstr($ra);
   $at->put($j+1,$xpos++,$at->Label(-text => "$sfield", -anchor => 'e'));
  my $dec=($ymin[$j]+$ymax[$j])/2.;
    $sfield=sexstr($dec);
   $at->put($j+1,$xpos++,$at->Label(-text => "$sfield", -anchor => 'e'));
 
  }
    
#    foreach my $i (0..$#newtab)  {
#      my $j=$ira->at($i);
      
#      my $xpos=0;
      
#      my $e;
      
#      ${$e}=$at->BrowseEntry(-width =>'30', -variable => \$newtab[$j][1]);
#    $frames[$i+1]=${$e};$at->put($i+1,$xpos++,${$e});
#if ($newtab[$j][1] ne $newtab[$j][0]) {$frames[$i+1]->insert("end",$newtab[$j][1])}
#$frames[$i+1]->insert("end", $newtab[$j][0]);

#foreach my $fc (0..1) {
#  my $field=$newtab[$j][$fc+3];
#  my $sfield=sexstr($field);
#  $newtab[$j][$fc+3]= $sfield;
#  $at->put($i+1,$xpos++,$at->Label(-text => "$sfield", -anchor => 'e'));
#}

##$at->put($i+1,$xpos++,$at->Label(-text => "$newtab[$j][2]", -anchor => 'e'));

#}

$frames[1]->focus;

$atab->waitWindow;


die;
    
    # ===================
    
    @newtab=();
    my @mra=();
    my @mdec=();
    
    foreach my $name (@onames)  {
      $nidx++;
      
      if ($DEBUG){  print "\n>> $name << \n"; }
      my @xflog=grep(/\b$name\b/i,@flog);
      my ($ra, $dec);
      my $nrows=$#xflog+1;
      my $coords=zeroes(float,$nrows,3);
      
      my $r=0;
      foreach my $row (@xflog) {
        my @fields=split(/$main::DBDEL/,$row);
        my $c=0;
        foreach my $field (@fields) {
	  if ($header[$c] eq 'nr') {set $coords,$r, 0, $field}
	  elsif ($header[$c] eq 'ra') {my @str=split(/:/,$field,3);
				       my $val=$str[0]+$str[1]/60.+$str[2]/3600.;
				       set $coords,$r, 1, $val;
				     }
	  elsif ($header[$c] eq 'dec') {my @str=split(/:/,$field,3);
					my $sig=1;
					if ($str[0]=~/-/) {$sig=-1};
					my $val=$sig*(abs($str[0])+$str[1]/60.+$str[2]/3600.);
					
					set $coords,$r, 2, $val;
					last}
	  $c++;
	}			#end loop on fields
	$r++;
      }				#end loop on rows of subsample
      
      # print "   Mean coordinates:  \n";
      
      my $c1= $coords->slice(':,(1)') ;
      my $c2= $coords->slice(':,(2)') ;
      
      
      my ($amin, $amax, $dmin, $dmax);
      
      $amin=min($c1); $amax=max($c1);
      $dmin=min($c2); $dmax=max($c2);
      
      
      my $ast=$thresh->at(0); 
      $amin-=$ast;
      $amax+=$ast;
      my $abins= int(($amax-$amin)/$ast+.5) ;
      
      my $dst=$thresh->at(1);
      $dmin-=$dst;
      $dmax+=$dst;
      my $dbins=int(($dmax-$dmin)/$dst+.5);
      
      #   print "$c1 $c2\n";
      if ($DEBUG){       print "$amin $amax $ast $abins, $dmin $dmax $dst $dbins\n";  }
      
      #Compute the histogram of the coordinates and find clusters
      my $h = histogram2d($c1, $c2, $ast, $amin ,$abins , $dst, $dmin, $dbins);
      my ($nch, $nrh)=dims($h);
      my $nz=which($h->clump(2)>0);
      my $nzy=rint($nz/$nch);
      my $nzx=($nz-$nzy*$nch);
      
      # $clust=cat $amin-$ast+$nzx*$ast , $dmin-$dst+$nzy*$dst;
      # Either we should build an array of hashes with new names & coordinates
      # or all info should be written to a line.......
      # first better because then it is easy to reloop and look for misidentifications
      # structure: name, un/changed, alfa, delta
      # ($new->)
      my $mod='-';		#Default: all ok....
      if (nelem($nzx) > 1) {$mod='W'} #more than one peak, something is suspicious......
      
      for my $clust (0..nelem($nzx)-1) {
	my $nname=$name;
	if ($clust>0) {$nname=$nname."#$clust"}
	my $clra= $amin+$ast*$nzx->at($clust);
	my $cldec=  $dmin+$dst*$nzy->at($clust);
	
	my $sc=which(($c1>= ($clra) & $c1 < ($clra+$ast)) & ($c2>= ($cldec) & $c2 < ($cldec+$dst))  );
	my $mra=median(index($c1,$sc));
	my $mdec=median(index($c2,$sc));
	if ($DEBUG) { print "$clra $ast $cldec $dst $sc  --> $mra $mdec\n";}
	push @newtab, [$nname, $nname, $mod, $mra, $mdec, index($coords->slice(':,(0)'),$sc)];
	push (@mra,$mra);
	push (@mdec,$mdec);
      }
      
    }				#end loop on names
    
    #convert to piddles and free memory
    my $mra=float(@mra);
    @mra=();
    my $mdec=float(@mdec);
    @mdec=();
    
    #And now, we loop again and find different clusters with same coordinates..................
    foreach my $i (0..$#newtab)  {
      # next if ($newtab[$i][2] ne 'W') ;  #uncomment to check only "suspicious" groups
      
      my $min_dra=9999;
      my $min_ddec=9999;
      my $guess=-1;
      foreach my $j (0..$#newtab)  {
	next if ($j==$i || $newtab[$j][2] ne '-'); #skip if the entry is the same or it is wrong 
	my $dra = abs($newtab[$i][3]- $newtab[$j][3] );
	my $ddec = abs( $newtab[$i][4]- $newtab[$j][4] );
	if ($dra <= $thresh->at(0) && $ddec <= $thresh->at(1))
	  {  if ($dra < $min_dra && $ddec < $min_ddec)  {
	    $min_dra=$dra; $min_ddec=$ddec; $guess=$j;$newtab[$i][2]='M';
	  }
	     if ($DEBUG) {print "found !! $newtab[$i][0] & $newtab[$j][0]\n"}
	   }
      }
      if ($guess>=0) {
	$newtab[$i][1]=$newtab[$guess][0];
	if ($DEBUG) {print ">>>>>> Most likely: $newtab[$guess][0]\n"}
      }
    }
    
    
    #The last effort: sort the @newtab array by alfa and delta
    
    #First sort by ra
    my  $ira=qsorti($mra);
    
    # now sort by dec
    
    my $rdim=nelem($mra);
    my $f=zeroes (ushort,$rdim);
    
    $mra=$mra->index($ira);
    $mdec=$mdec->index($ira);
    
    
    my $i;
    
    for ($i=0;$i<$rdim;$i++) {
      
      #skip if flagged
      if ($f->at($i)==1) {next}
      
      #find all obs with same ra and flag them
      my $sra=$mra->at($i);
      my $ridx=which ($mra==$sra);
      (my $tmp=$f->index($ridx)) .= 1;
      
      #extract decs
      my $xdec=$mdec->index($ridx);
      my $ddim=nelem($xdec);
      
      #skip if it is only one element
      if ($ddim<=1) {next}
      
      #sort
      my $didx=qsorti($xdec);
      
      #change the order of the sorting index
      ($tmp = $ira->index($ridx)) .= $tmp->index($didx);
      
    }
    
    
    
    #interactively change @newtab
    
    $top=MainWindow->new(-name => 'Check DB', -title=>"Check DB");
    my $atab = $top;
    $atab->title('Clusters of names');
    
    # Use a scrollable table (useful for task with many parameters)
    
    my $at  = $atab->Table(-scrollbars => 'e',-fixedrows => 1, -rows => 20);
    $at->pack(-expand=> 1, -fill => 'both');
    
    my $c=0;
    
    $at->put(0,$c++,$at->Label(-text=>"Name",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"RA",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"DEC",-anchor=>'w'),-relief => 'raised');
    $at->put(0,$c++,$at->Label(-text=>"Flag",-anchor=>'w'),-relief => 'raised');
    
    $at->configure(-columns => $c) if ($c > $at->cget('-columns'));
    
    my $buttons = $atab->Frame();
    $buttons->pack(-side => 'bottom', -pady => '2', -anchor => 'center');
    #my $buttons_clear = $buttons->Button(-text => 'Clear');
    #my $buttons_load = $buttons->Button(-text => 'Load');
    my $buttons_apply = $buttons->Button(-text => 'Apply');
    $buttons_apply->configure(-command => \&apply_changes);
    #my $buttons_savedef = $buttons->Button(-text => 'Save def.');
    my $buttons_exit = $buttons->Button(-text => 'Close');
    $buttons_exit->configure(-command =>  (sub {$atab->destroy}));
    #$buttons_skip = $buttons->Button(-text => 'Next');
    
    #If not interactive mode, windows are only used to show the parameters (read only)
    #if ($main::redmode eq "interactive") {
    #    $buttons_clear->pack(-side => 'left', -padx => '2');
    #    $buttons_load->pack(-side => 'left', -padx => '2');
    $buttons_apply->pack(-side => 'left', -padx => '2');
    #    $buttons_savedef->pack(-side => 'left', -padx => '2');
    $buttons_exit->pack(-side => 'left', -padx => '2');
    #$buttons_skip->pack(-side => 'left', -padx => '2');
    #  }
    
    
    foreach my $i (0..$#newtab)  {
      my $j=$ira->at($i);
      
      my $xpos=0;
      
      my $e;
      
      ${$e}=$at->BrowseEntry(-width =>'30', -variable => \$newtab[$j][1]);
    $frames[$i+1]=${$e};$at->put($i+1,$xpos++,${$e});
if ($newtab[$j][1] ne $newtab[$j][0]) {$frames[$i+1]->insert("end",$newtab[$j][1])}
$frames[$i+1]->insert("end", $newtab[$j][0]);

foreach my $fc (0..1) {
  my $field=$newtab[$j][$fc+3];
  my $sfield=sexstr($field);
  $newtab[$j][$fc+3]= $sfield;
  $at->put($i+1,$xpos++,$at->Label(-text => "$sfield", -anchor => 'e'));
}

$at->put($i+1,$xpos++,$at->Label(-text => "$newtab[$j][2]", -anchor => 'e'));

}

$frames[1]->focus;

$atab->waitWindow;

}

sub apply_changes
  {
    
    
    #replace names and coords in the dblog array (should be the last step....)
    foreach my $i (0..$#newtab)  {
      
      my $p=$newtab[$i][$#{$newtab[$i]}] ;
      
      #update the database
      foreach my $j (0..nelem($p)-1) {
	my $idx=$p->at($j);
	if ($DEBUG) {print ">>>>>>> Reading $idx ($newtab[$i][1])/ ";}
	my $row= $log[$idx];
	my @fields=split(/$main::DBDEL/,$row);
        my $c=0;
        my @nrow=();
	
	my @warnings=();
	
        foreach my $field (@fields) {
	  if ($header[$c] eq "object") {$field =~ tr/ //d;
					push @nrow, $newtab[$i][1];
					if (lc($field) ne lc($newtab[$i][1])) {
					  push @warnings, "# Name changed (was $field)";
					}
				      }
	  elsif ($header[$c] eq "ra") {push @nrow, $field} #we display the TRUE coordinates 
	  elsif ($header[$c] eq "dec") {push @nrow, $field}
	  elsif ($header[$c] eq "warnings") {push @warnings, $field}	
	  else {push @nrow, $field}
	  $c++;
	}			#end loop on fields
	my $warning =join " ", @warnings;
	my $nrow=join "$main::DBDEL", @nrow,$warning;
	if ($DEBUG) {print "\n###### idx -  $log[$idx] -> $nrow ##### \n"};
	
	$log[$idx]=$nrow ; 
      }
      
      if ($DEBUG) {print "\n";}
      
    }
    
    #ok, dump the new @log to the file
    
    open (NEWDB, ">$odbfile");
    
    my $now=&main::mkdate;
    print NEWDB "<DATE> $now </DATE>\n";
    
    print NEWDB "<HEADER>\n";
    my $leg=join $main::DBDEL, $log[0];
    shift @log;
    print NEWDB "$leg\n";
    print NEWDB "</HEADER>\n";
    
    foreach my $row (@log) {chomp($row); print NEWDB "$row\n"; }
    
    close (NEWDB);
    
    $main::top->Dialog(
		       -title  => 'CheckDB',
		       -text   => "The file $odbfile has been made",
		       -bitmap => 'warning',
		      )->Show;
    
  }

sub hscan
  {
   my $h=shift;
   my ($amin, $dmin, $ast, $dst, $nelems)=@_;

 
 
     if ($ast <  $thresh->at(0)/5 && $dst <  $thresh->at(1)/5)
       {
 #  warn "LOOP completed: $nelems / $ast, $dst  ($thresh)";
push @xmin, $lxmin; push @xmax, $lxmax;
push @ymin, $lymin; push @ymax, $lymax;
push @nels, $nelems;
   return 
       }

   my $nast=$ast/5;
   my $ndst=$dst/5;
     
   my ($xnz, $ynz) = whichND($h !=0);
   my $dnz=$ynz->getdim(0)-1;
   my $hx=$xnz*$ast+$amin;
   my $hy=$ynz*$dst+$dmin;

#warn "$xnz\n $ynz\n $hx \n $hy";  
   foreach my $j (0..$dnz) {   

     my $xamin=$hx->at($j); 
     my $xamax=$hx->at($j)+$ast; 

     my $xdmin=$hy->at($j); 
     my $xdmax=$hy->at($j)+$dst; 

     my $idx=which($c1>=$xamin & $c1<$xamax & $c2>=$xdmin & $c2<$xdmax); 
#warn "=========== $xamin $xamax $xdmin $xdmax -> ", $idx->nelem;
     next if ($idx->nelem < 1);
     my $xc1=$c1->index($idx);
     my $xc2=$c2->index($idx);  

     my $abins= int(($xamax-$xamin)/$nast+.5) ;
     my $dbins= int(($xdmax-$xdmin)/$ndst+.5);    
  
#warn "mk histogram for:  $nast, $xamin ,$abins , $ndst, $xdmin, $dbins";
     my $h1 = histogram2d($xc1, $xc2, $nast, $xamin ,$abins , $ndst, $xdmin, $dbins);
     
#  warn "new level with $nast $ndst\n";
$lxmin=$xamin; $lxmax=$xamax;
$lymin=$xdmin; $lymax=$xdmax;

      hscan($h1, $xamin, $xdmin, $nast, $ndst, $idx->nelem);
   }
#warn "LOOP over";
  }



sub sexstr {
  my $value=shift;
  my $sig=$value/abs($value);
  my $f1=int($value);
  my $f3=abs($value-$f1)*60.;
  my $f2=int($f3);
   $f3=($f3-$f2)*60;
   my $if3=int($f3);
    my $value=sprintf ("%02d:%02d:%02d.%.0f",$f1,$f2,$if3,100*($f3-$if3));
    if ($sig==-1 && $f1 ==0)  {$value='-'.$value}
   elsif ($f1 !~ /^-/ && $f1 >=0) {$value='+'.$value}
   
   return($value);
}

sub FName 
  {
my ($names, $idx)=@_;

my $cn=lc($$names[$idx->at(0)]);
return $cn if ($idx->nelem==1);
 
  foreach my $k (1..$idx->nelem-1) {
      $_=$cn;
      my $m=lc($$names[$idx->at($k)]);
      my $r=eval "tr/$m//cd";
      if ($r<length($cn)) {$cn=$_}
    }
return $cn;
  }

1;
