
use strict;

use vars qw(@SConf $SWARPconf $SWbin %SwarpCVals);

sub InitSWARP
{

$SWbin='/u2/bertin/bin/swarp';

$SWARPconf='default.swarp';

@SConf=(

    {
   COMM =>  ' Default configuration file for SWarp 1.33# EB 30/04/2002',
    }, 

    {
   COMM =>  '----------------------------------- Output -----------------------------------',
    }, 

    {
   KEY  =>  'IMAGEOUT_NAME',
   VAL  =>  'coadd.fits',
   COMM =>  ' Output filename',
    }, 

    {
   KEY  =>  'WEIGHTOUT_NAME',
   VAL  =>  'coadd.weight.fits',
   COMM =>  ' Output weight-map filename',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'HEADER_ONLY',
   VAL  =>  'N',
   COMM =>  ' Only a header as an output file (Y/N)?',
    }, 

    {
   KEY  =>  'HEADER_SUFFIX',
   VAL  =>  '.head',
   COMM =>  ' Filename extension for additional headers',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   COMM =>  '------------------------------- Input Weights --------------------------------',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'WEIGHT_TYPE',
   VAL  =>  'NONE',
   COMM =>  ' "BACKGROUND","MAP_RMS","MAP_VARIANCE"',
    }, 

    {
   COMM =>  ' or "MAP_WEIGHT"',
    }, 

    {
   KEY  =>  'WEIGHT_SUFFIX',
   VAL  =>  '.weight.fits',
   COMM =>  ' Suffix to use for weight-maps',
    }, 

    {
   KEY  =>  'WEIGHT_IMAGE',
   COMM =>  ' Weightmap filename if suffix not used',
    }, 

    {
   COMM =>  ' (all or for each weight-map)',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   COMM =>  '------------------------------- Co-addition ----------------------------------',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'COMBINE',
   VAL  =>  'Y',
   COMM =>  ' Combine resampled images (Y/N)?',
    }, 

    {
   KEY  =>  'COMBINE_TYPE',
   VAL  =>  'MEDIAN',
   COMM =>  ' "MEDIAN","AVERAGE","MIN","MAX"',
    }, 

    {
   COMM =>  ' or "WEIGHTED"',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   COMM =>  '-------------------------------- Astrometry ----------------------------------',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'CELESTIAL_TYPE',
   VAL  =>  'NATIVE',
   COMM =>  ' "NATIVE", "EQUATORIAL", "GALACTIC",',
    }, 

    {
   COMM =>  ' or "ECLIPTIC"',
    }, 

    {
   KEY  =>  'PROJECTION_TYPE',
   VAL  =>  'TAN',
   COMM =>  ' Any WCS projection code or "NONE"',
    }, 

    {
   KEY  =>  'CENTER_TYPE',
   VAL  =>  'ALL',
   COMM =>  ' "MANUAL", "ALL" or "MOST"',
    }, 

    {
   KEY  =>  'CENTER',
   VAL  =>  '00:00:00.0, +00:00:00.0',
   COMM =>  ' Coordinates of the image center',
    }, 

    {
   KEY  =>  'PIXELSCALE_TYPE',
   VAL  =>  'MEDIAN',
   COMM =>  ' "MANUAL", "FIT", "MIN", "MAX"',
    }, 

    {
   COMM =>  ' or "MEDIAN"',
    }, 

    {
   KEY  =>  'PIXEL_SCALE',
   VAL  =>  '0.0',
   COMM =>  ' Pixel scale',
    }, 

    {
   KEY  =>  'IMAGE_SIZE',
   VAL  =>  '0',
   COMM =>  ' Image size (0 = AUTOMATIC)',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   COMM =>  '-------------------------------- Resampling ----------------------------------',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'RESAMPLE',
   VAL  =>  'Y',
   COMM =>  ' Resample input images (Y/N)?',
    }, 

    {
   KEY  =>  'RESAMPLE_DIR',
   VAL  =>  '.',
   COMM =>  ' Directory path for resampled images',
    }, 

    {
   KEY  =>  'RESAMPLE_SUFFIX',
   VAL  =>  '.resamp.fits',
   COMM =>  ' filename extension for resampled images',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'RESAMPLING_TYPE',
   VAL  =>  'LANCZOS3',
   COMM =>  ' "NEAREST","BILINEAR","LANCZOS2",',
    }, 

    {
   COMM =>  ' "LANCZOS3" or "LANCZOS4" (1 per axis)',
    }, 

    {
   KEY  =>  'OVERSAMPLING',
   VAL  =>  '0',
   COMM =>  ' Oversampling in each dimension',
    }, 

    {
   COMM =>  ' (0 = automatic)',
    }, 

    {
   KEY  =>  'INTERPOLATE',
   VAL  =>  'N',
   COMM =>  ' Interpolate bad input pixels (Y/N)?',
    }, 

    {
   COMM =>  ' (all or for each image)',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'FSCALASTRO_TYPE',
   VAL  =>  'FIXED',
   COMM =>  ' "NONE" or "FIXED"',
    }, 

    {
   KEY  =>  'FSCALE_KEYWORD',
   VAL  =>  'FSCALE',
   COMM =>  ' FITS keyword for the multiplicative',
    }, 

    {
   COMM =>  ' factor to apply to each input image',
    }, 

    {
   KEY  =>  'FSCALE_DEFAULT',
   VAL  =>  '1.0',
   COMM =>  ' Default FSCALE value if not in header',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'GAIN_KEYWORD',
   VAL  =>  'GAIN',
   COMM =>  ' FITS keyword for eff. gain (e-/ADU)',
    }, 

    {
   KEY  =>  'GAIN_DEFAULT',
   VAL  =>  '0.0',
   COMM =>  ' Default gain if no FITS keyword found',
    }, 

    {
   COMM =>  ' 0 = infty (all or for each image)',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   COMM =>  '--------------------------- Background subtraction ---------------------------',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'SUBTRACT_BACK',
   VAL  =>  'Y',
   COMM =>  ' Subtraction sky background (Y/N)?',
    }, 

    {
   COMM =>  ' (all or for each image)',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'BACK_TYPE',
   VAL  =>  'AUTO',
   COMM =>  ' "AUTO" or "MANUAL"',
    }, 

    {
   COMM =>  ' (all or for each image)',
    }, 

    {
   KEY  =>  'BACK_DEFAULT',
   VAL  =>  '0.0',
   COMM =>  ' Default background value in MANUAL',
    }, 

    {
   COMM =>  ' (all or for each image)',
    }, 

    {
   KEY  =>  'BACK_SIZE',
   VAL  =>  '128',
   COMM =>  ' Background mesh size (pixels)',
    }, 

    {
   COMM =>  ' (all or for each image)',
    }, 

    {
   KEY  =>  'BACK_FILTERSIZE',
   VAL  =>  '3',
   COMM =>  ' Background map filter range (meshes)',
    }, 

    {
   COMM =>  ' (all or for each image)',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   COMM =>  '------------------------- Virtual memory management --------------------------',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'VMEM_DIR',
   VAL  =>  '.',
   COMM =>  ' Directory path for swap files',
    }, 

    {
   KEY  =>  'VMEM_MAX',
   VAL  =>  '2047',
   COMM =>  ' Maximum amount of virtual memory (MB)',
    }, 

    {
   KEY  =>  'MEM_MAX',
   VAL  =>  '128',
   COMM =>  ' Maximum amount of usable RAM (MB)',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   COMM =>  '------------------------------ Miscellaneous ---------------------------------',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'DELETE_TMPFILES',
   VAL  =>  'Y',
   COMM =>  ' Delete temporary resampled FITS files',
    }, 

    {
   COMM =>  ' (Y/N)?',
    }, 

    {
   KEY  =>  'WRITE_FILEINFO',
   VAL  =>  'N',
   COMM =>  ' Write information about each input',
    }, 

    {
   COMM =>  ' file in the output image header?',
    }, 

    {
   KEY  =>  'VERBOSE_TYPE',
   VAL  =>  'NORMAL',
   COMM =>  ' "QUIET","NORMAL" or "FULL"',
    }, 

    {
   KEY  =>  ' ',
    }, 

    {
   KEY  =>  'NTHREADS',
   VAL  =>  '1',
   COMM =>  ' 1 single thread',
    }, 

     );

}


sub mkSWConf
{
open (FILE, ">$SWARPconf"); 
my $entry;
foreach $entry (@SConf) {

  my $nokey=0;
   if (exists $entry->{KEY}) {
      my $key=$entry->{KEY};
      print FILE "$key\t";
      if (exists $SwarpCVals{$key}) {
           print FILE "$SwarpCVals{$key}\t";
	}  
      elsif (exists $entry->{VAL}) {
           print FILE "$entry->{VAL}\t";
	}   
    } else {$nokey=1}
    if (exists $entry->{COMM}) {
        my $comm=$entry->{COMM};
         if ($nokey==1 && $comm !~ '--') {print FILE "\t\t\t"}
	 
           print FILE "#$comm";
	}   
  print FILE "\n";	
   }
   
 close (FILE);  
   
 }

1;



