#!/usr/bin/perl

use PDL;

use Getopt::Long;
use Common::fitsio;
use strict;
use SWARPconf;
use SEXconf;

my $SWARP='swarp';
my $MPSWARP='mpswarp';
my $MP=0;

use strict;

use Tie::IxHash;

my $REDOMASKS=1;

my $sconf='';

my %optctl = ('redomasks'=>\$REDOMASKS);
&GetOptions (\%optctl, "h",
	     "fits_dir|fdir=s",
	     "hdr_dir|hdir=s",
	     "swarp_dir|sdir=s",
             "sconf=s"=>\$sconf, 
	     "list|l=s@",
	     "mlist|m=s",
	     "name|n=s",
	     "crfilter|crf=s",	
             "redomasks|rm!",
             "combine=s",
	     "mp=i"=>\$MP
	    );


if ($optctl{h}) {
print "Usage: mkswarp -l[ist] =  (the list of fits files to be coadded)\n";
print "               -m[list] = (the list of mask files to be used)\n";
print "               -f[its_]dir = (the path to be added to list)\n";
print "               -h[dr_]dir = (the directory with headers)\n";
print "               -s[warp_]dir = (the output directory)\n";
print "               -n[ame] = (the name of the coadded files)\n";
print "               -combine = A(verage)|M(edian)|W(eighted)\n";
print "               -crf[ilter] = (the name of the filter to detect cosmic rays)  \n";
print "               -mp=2 = the number of threads if MP is used\n"; 
print 'e.g.: mkswarp.pl -f=/data/virmosu/mrad/DELETE -l=ima_split.lst -hdir=global -sdir=swarp -n=coadd -m=mask.lst',"\n";
die "\n";
}

my $hdir=$optctl{hdr_dir};
my $sdir=$optctl{swarp_dir};
my $mlist="";
$mlist=$optctl{mlist} if (defined $optctl{mlist});

my $name="coadd";
$name=$optctl{name} if (defined $optctl{name});

my $combtype='';

if (defined $optctl{combine}) {
  $combtype=lc(substr($optctl{combine},0,1));
  if ($combtype eq 'm') {$combtype='MEDIAN'}
  elsif ($combtype eq 'a') {$combtype='AVERAGE'}
  elsif ($combtype eq 'w') {$combtype='WEIGHTED'}
  else { die "-combine: $combtype not recognized !\n";}
}


my @inlists=();

if (defined $optctl{'list'}) {
  @inlists=@{$optctl{'list'}};
  my @c=();
  foreach my $tl (@inlists) {

    if (rindex($tl, '*')==-1) {
       push @c, $tl;
       next;
    }
     my @xl=glob($tl);

    push @c, @xl; 
  }
 @inlists=@c;   
}
else {print "Input list must be defined !!\n"; die}

my $XDIR=1;
if (! -e $sdir) {
   system ("mkdir $sdir") ;
   $XDIR=0;
 }

my $fdir='.';
if (defined $optctl{fits_dir}) {$fdir= $optctl{fits_dir}};

#my $combtype='MEDIAN';

#---------------------

#my %opt;
my $crfilt;

if (defined $optctl{crfilter}) {
  $combtype="WEIGHTED" if ($combtype eq '');
  
  $crfilt= $optctl{crfilter};
    &InitSEx();  #default values

if (! -e $crfilt) {die "CR filter >$crfilt< not found !!\n"}
    foreach my $entry (@SExConf) {  
      if (exists $entry->{KEY}) {
	my $key=$entry->{KEY};
	
	if ($key eq 'DETECT_MINAREA') {
	  $entry->{VAL}=1;
	}
	elsif ($key eq 'DETECT_THRESH') {
	  $entry->{VAL}=6;
	}
	elsif ($key eq 'ANALYSIS_THRESH') {
	  $entry->{VAL}=100;
	}
	elsif ($key eq 'FILTER') {
	  $entry->{VAL}='Y';
	}
	elsif ($key eq 'FILTER_NAME') {
	  $entry->{VAL}=$crfilt;
	}
	elsif ($key eq 'FILTER_THRESH') {
	  $entry->{VAL}=3;
	}
	elsif ($key eq 'DETECT_MINAREA') {
	  $entry->{VAL}=1;
	}
	elsif ($key eq 'CLEAN') {
	  $entry->{VAL}='N';
	}	
	elsif ($key eq 'MASK_TYPE') {
	  $entry->{VAL}='NONE';
	}	
	elsif ($key eq 'BACK_SIZE') {
	  $entry->{VAL}=64;
	}
	elsif ($key eq 'BACK_FILTERSIZE') {
	  $entry->{VAL}=5;
	}
        elsif ($key eq 'CHECKIMAGE_TYPE') {
	  $entry->{VAL}='SEGMENTATION';
	}
	elsif ($key eq 'CHECKIMAGE_NAME') {
	  $entry->{VAL}='cr.fits';
	}
       }
    }
    

  #Make the configuration files
 
      &mkSExConf();      #write the configuration and parameter files
#      &mkSExPars();

  open (PFILE, ">sex.pars");
     print PFILE "NUMBER\n\n";
  close (PFILE);

    }
#---------------------




if ($sconf eq '') {

&InitSWARP();

$combtype='MEDIAN' if ($combtype eq '');

$SwarpCVals{VMEM_DIR}=$sdir;
$SwarpCVals{RESAMPLE_DIR}=$sdir;
$SwarpCVals{COMBINE_TYPE}=$combtype;

if ($mlist eq '') {
  $SwarpCVals{WEIGHT_TYPE}='NONE';
} else { $SwarpCVals{WEIGHT_TYPE}='MAP_WEIGHT'}


if ($MP>1) { 
   $SwarpCVals{NTHREADS}=$MP
}



&mkSWConf;

$sconf='default.swarp'
}


 $SWARP=$MPSWARP if ($MP>1);

my @files=();

foreach my $list (@inlists) {

print "------ $list\n";

if (! -e $list) {die "List >$list< not found !!!!";  }

open (LIST, "<$list");

  while (<LIST>) { 
     chomp $_;  
     
     my $fits=$fdir.'/'.$_;
     if (! -e $fits) {print "Image $fits not found !!\n"; next}
     
      my %fh=&fitsio::rhead($fits);
      
     my @s=split (/\//, $fits);
     my $root=$s[-1];

     system ("rm $sdir/$root") if ($XDIR==1);# if (-l "$sdir/$root");
     die ("ERROR: FITS file $fits not found !!!!!! \n") if (! -e  $fits);
     system ("ln -s $fits $sdir/$root"); 

     push @files,  "$sdir/$root";

     @s=split (/\./, $root);
     $s[-1]='head';
     my $head =join '.', @s;
     my $dhead=$hdir.'/'.$head;

     die ("ERROR: Header $dhead not found !!!!!! \n") if (! -e  $dhead);

     system ("rm $sdir/$head")  if ($XDIR==1); #if (-l "$sdir/$head");
    system ("ln -s $dhead $sdir/$head");

    
     my $imid=$fh{IMAGEID};
#    my $imask;

open (MASK, "<$mlist"); 

     while (<MASK>) {
       chomp ($_);
       my $imask=$_;

       my %mh=&fitsio::rhead($imask);
       my $mid=$mh{IMAGEID};
       $mid=~ tr/\'//d;

       if ($mid==$imid) {
	 $s[-1]='weight.fits';
	 my $mask=join'.', @s;
	 
	 system ("rm $sdir/$mask") if ($XDIR==1 && -e "$sdir/$mask" && $REDOMASKS==1);

         if (! -e "$sdir/$mask") {

#goood place to create masks......
if ($crfilt ne "") {

warn "adding cosmics to masks using $crfilt.....";

&RunSEx($files[-1], 'del.coo');


  my %ch=&fitsio::rhead("cr.fits");
  my ($cmask, $ret)=&fitsio::rfits("cr.fits"); 
  
  my %mh;
  tie (%mh, 'Tie::IxHash'); 

  %mh=&fitsio::rhead($imask);
  my ($pmask, $ret)=&fitsio::rfits($imask);

  my ($xm, $ym)=whichND($cmask!=0);
  my $fim=$pmask->index2d($xm, $ym);
  $fim.=0;
  $mh{COSMIC}="\'$crfilt\'";


  my $wret=&fitsio::wfits($pmask, "$sdir/$mask", \%mh);

 # system ("rm del.coo") if (-e "del.coo");

}
else {
#   print "linking $imask to $sdir/$mask\n";
	 system ("ln -s $imask $sdir/$mask");
}
  }
	 last;
       }
     }
close (MASK);    

   }

close (LIST);

}

#my $sfile=$sdir.'/'.$name.'.fits';
#my $wfile=$sdir.'/'.$name.'.weight.fits';

my $sfile=$name.'.fits';
my $wfile=$name.'.weight.fits';


print "will swarp files to $sfile ($wfile)\n";

print ("$SWARP -c $sconf -IMAGEOUT_NAME $sfile -WEIGHTOUT_NAME $wfile -COMBINE_TYPE=$combtype");



system ("$SWARP -c $sconf @files -IMAGEOUT_NAME $sfile -WEIGHTOUT_NAME $wfile");


#print "@files\n";


